(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.Compatibility.CompatArray

let (-) = Microsoft.FSharp.Primitives.Operators.op_IntegerSubtraction
let (+) = Microsoft.FSharp.Primitives.Operators.op_IntegerAddition

(* Define the primitive operations. *)
(* Note: the "type" syntax is for the type parameter for inline *)
(* polymorphic IL. This helps the compiler inline these fragments, *)
(* i.e. work out the correspondence between IL and F# type variables. *)
let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 length (arr: 'a[]) =  (# "ldlen" arr : int)  

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 get (arr: 'a[]) (n:int) =  (# "ldelem.any !0" type ('a) arr n : 'a)

#if GENERICS
let set (arr: 'a[]) (n:int) (x:'a) =  (# "stelem.any !0" type ('a) arr n x)
#else
// The above do not work for value types when compiling without generics, since 
// ILX is not able to erase the use of stelem.any
let inline set (arr: 'a[]) (n:int) (x:'a) =  (# "stobj !0" type ('a) (# "ldelema !0" type ('a) arr n : System.UIntPtr) x)
#endif

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 zero_create (n:int) = (# "newarr !0" type ('a) n : 'a[])

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 make  (n:int) x =
  let arr = zero_create n in 
  for i = 0 to n - 1 do 
    (set arr i x)
  done;
  arr

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 create (n:int) x = make n x

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 init (n:int) f = 
  let arr = zero_create n  in 
  for i = 0 to n - 1 do 
    set arr i (f i)
  done;
  arr

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 nonnull x = match x with [] -> false | _ -> true

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 concat arrs =
  let rec list_fold_left f acc l = 
    match l with 
      [] -> acc
    | (h::t) -> list_fold_left f (f acc h) t in
  let lenall = list_fold_left (fun x y -> x + length y) 0 arrs in 
  let res = zero_create lenall in 
  let mutable n = 0 in
  let mutable curr = arrs in 
  while nonnull curr do
    let arr::t = curr in
    let len = length arr in 
    for i = 0 to len - 1 do 
      set res (n + i) (get arr i )
    done;
    n <- n + len;
    curr <- t;
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 append arr1 arr2 = concat [arr1; arr2]

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 sub arr (start:int) (len:int) =
  let res = zero_create len   in
  for i = 0 to len - 1 do 
    set res i (get arr (start + i))
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 fill arr (start:int) (len:int) x =
  for i = start to start + len - 1 do 
    (set arr i x)
  done

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 copy arr = concat [arr]

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 blit arr1 (start1:int) arr2 (start2:int) (len:int) =
  for i = 0 to len - 1 do 
    set arr2 (start2+i) (get arr1 (start1 + i))
  done

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 to_list arr =
  let len = length arr in 
  let mutable res = [] in 
  for i = len - 1 downto 0 do 
    res <- (get arr i) :: res
  done;
  res
  
let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 nonempty x = match x with [] -> false | _ -> true

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 list_length l = 
  let mutable res = 0 in 
  let mutable curr = l in 
  while (nonempty curr) do 
    res <- res + 1;
    curr <- curr.(::).1
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 of_list l =
  let len = list_length l in 
  let res = zero_create len  in 
  let mutable lref = l in 
  for i = 0 to len - 1 do 
    let h :: t = lref in 
    set res i h;
    lref <- t
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 iter f arr =
  let len = length arr in 
  for i = 0 to len - 1 do 
    f (get arr i)
  done

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 map f arr =
  let len = length arr in 
  let res = zero_create len in 
  for i = 0 to len - 1 do 
    set res i (f (get arr i))
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 iteri f arr =
  let len = length arr in 
  for i = 0 to len - 1 do 
    f i (get arr i)
  done


let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 mapi f arr =
  let len = length arr in 
  let res = zero_create len  in 
  for i = 0 to len - 1 do 
    set res i  (f i (get arr i))
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 fold_left f acc arr =
  let mutable res = acc in 
  let len = length arr in 
  for i = 0 to len - 1 do 
    res <- f res (get arr i)
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 fold_right f arr acc =
  let mutable res = acc in 
  let len = length arr in 
  for i = len - 1 downto 0 do 
    res <- f (get arr i) res
  done;
  res


module Array = Microsoft.FSharp.Primitives.ArrayPrimitives

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 of_array x =
  let res = zero_create (Array.length x) in
  for i = 0 to Array.length x - 1 do
    set res i (Array.get x i)
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 to_array x =
  let res = Array.zero_create (length x) in
  for i = 0 to length x - 1 do
    Array.set res i (get x i)
  done;
  res

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 foreach x f  = iter f x

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 transform x f  = map f x

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 foreachi x f  = iteri f x

let
#if CLI_FRAMEWORK_AT_MOST_RTM
inline
#endif
 transformi x f  = mapi f x
