(*==========================================================================
 * The type signatures in this file were automatically generated from the 
 * implementation.  Documentation (c) Microsoft Corporation 2004.
 *
 * All code and documentation in the interface and implementation of 
 * this module has been written 'from scratch'.  The interface to the module 
 * is similar to that found in versions of other ML implementations, 
 * but is not an exact match.  The type signatures in this interface
 * are an edited version of those generated automatically by running 
 * "bin\fsc.exe -i" on the implementation file.
 *=========================================================================*)

(*==========================================================================
 * Psuedo-polymorphic arrays.
 *
 * For any particular 'a , the "'a[]" type is type-compatible with 
 * the corresponding C# array type.  This module contains operations to 
 * manipulate these values.  These can be used on any version of the
 * .NET CLR.  See the 'Advanced' section of the manual for why array
 * operations differ in various versions of the CLR.
 *=========================================================================*)
 
module Microsoft.FSharp.Compatibility.CompatArray

#if GENERICS

///Build a new array that contains the elements of the first array followed by the elements of the second array
val append: 'a[] -> 'a[] -> 'a[]

///Read a range of elements from the first array and write them into the second.
val blit: 'a[] -> int -> 'a[] -> int -> int -> unit

///Build a new array that contains the elements of each of the given list of arrays
val concat: List< 'a[] > -> 'a[]

///Build a new array that contains the elements of the given array
val copy: 'a[] -> 'a[]

///Create a new array of the given length, each entry of which holds the given element
val create: int -> 'a -> 'a[]

///Fill a range of an array with the given element
val fill: 'a[] -> int -> int -> 'a -> unit

///Apply a function to each element of the collection, threading an 'accumulator' argument
///through the computation. If the elements are "i0...iN" then computes "f (... (f s i0)...) iN"
val fold_left: ('a -> 'b -> 'a) -> 'a -> 'b[] -> 'a

///Compute f (get arr n) (...(f (get arr n) s))
val fold_right: ('a -> 'b -> 'b) -> 'a[] -> 'b -> 'b

///Fetch the given element of the array, indexed by 0
val get: 'a[] -> int -> 'a

///Build a new array whose elements are f 0, ... f N
val init: int -> (int -> 'a) -> 'a[]

///Execute the given function for each element of the given array
val iter: ('a -> unit) -> 'a[] -> unit

///Execute the given function for each element of the given array
val foreach: 'a[] -> ('a -> unit) -> unit

///Execute the given function for each element of the given array.  The integer
///index indicates the index of the element.
val iteri: (int -> 'a -> unit) -> 'a[] -> unit

///Execute the given function for each element of the given array.  The integer
///index indicates the index of the element.
val foreachi: 'a[] -> (int -> 'a -> unit) -> unit

///The length of the array
val length: 'a[] -> int

///Build a new array of the given length, each entry of which holds the given element.
///Same as create.
val make: int -> 'a -> 'a[]

///Build a new array whose elements are the results of applying the given function
///to each of the elements of the array.
val map: ('a -> 'b) -> 'a[] -> 'b []

///Build a new array whose elements are the results of applying the given function
///to each of the elements of the array. The integer
///index passed to the funciton indicates the index of the element.
val mapi: (int -> 'a -> 'b) -> 'a[] -> 'b[]

///Build a new array whose elements are the results of applying the given function
///to each of the elements of the array.
val transform: 'a[] -> ('a -> 'b) -> 'b[]

///Build a new array whose elements are the results of applying the given function
///to each of the elements of the array. The integer
///index passed to the funciton indicates the index of the element.
val transformi: 'a[] -> (int -> 'a -> 'b) -> 'b[]

///Build a .NET compatible array from the F# array.  Note that F# arrays and .NET
///compatible arrays are only different types when using .NET 1.0 or 1.1
val of_array: 'a array -> 'a[]

///Convert the given list to a .NET compatible array suitable for passing to .NET functions
val of_list: List< 'a > -> 'a[]

///Set the given element of the array
val set: 'a[] -> int -> 'a -> unit

///Build a new .NET compatible array that contains the given subrange specified by
///starting index and length 
val sub: 'a[] -> int -> int -> 'a[]

///Build an F# array from the given .NET compatible array.  Note that F# arrays and .NET
///compatible arrays are only different types when using .NET 1.0 or 1.1
val to_array: 'a[] -> 'a array

///Convert the given .NET compatible array to a list
val to_list: 'a[] -> List< 'a >

///Create a new array filled with zero (null or default) values.  Warning: Use of this
///function may result in subsequent exceptions if the elements of the array
///are accessed before initialized.
val zero_create: int -> 'a[]

#else

///Build a new array that contains the elements of the first array followed by the elements of the second array
val inline append: $a[] -> $a[] -> $a[]

///Read a range of elements from the first array and write them into the second.
val inline blit: $a[] -> int -> $a[] -> int -> int -> unit

///Build a new array that contains the elements of each of the given list of arrays
val inline concat: List< $a[] > -> $a[]

///Build a new array that contains the elements of the given array
val inline copy: $a[] -> $a[]

///Create a new array of the given length, each entry of which holds the given element
val inline create: int -> $a -> $a[]

///Fill a range of an array with the given element
val inline fill: $a[] -> int -> int -> $a -> unit

///Apply a function to each element of the collection, threading an 'accumulator' argument
///through the computation. If the elements are "i0...iN" then computes "f (... (f s i0)...) iN"
val inline fold_left: ('a -> $b -> 'a) -> 'a -> $b[] -> 'a

///Compute f (get arr n) (...(f (get arr n) s))
val inline fold_right: ($a -> 'b -> 'b) -> $a[] -> 'b -> 'b

///Fetch the given element of the array, indexed by 0
val inline get: $a[] -> int -> $a

///Build a new array whose elements are f 0, ... f N
val inline init: int -> (int -> $a) -> $a[]

///Execute the given function for each element of the given array
val inline iter: ($a -> unit) -> $a[] -> unit

///Execute the given function for each element of the given array
val inline foreach: $a[] -> ($a -> unit) -> unit

///Execute the given function for each element of the given array.  The integer
///index indicates the index of the element.
val inline iteri: (int -> $a -> unit) -> $a[] -> unit

///Execute the given function for each element of the given array.  The integer
///index indicates the index of the element.
val inline foreachi: $a[] -> (int -> $a -> unit) -> unit

///The length of the array
val inline length: $a[] -> int

///Build a new array of the given length, each entry of which holds the given element.
///Same as create.
val inline make: int -> $a -> $a[]

///Build a new array whose elements are the results of applying the given function
///to each of the elements of the array.
val inline map: ($a -> $b) -> $a[] -> $b []

///Build a new array whose elements are the results of applying the given function
///to each of the elements of the array. The integer
///index passed to the funciton indicates the index of the element.
val inline mapi: (int -> $a -> $b) -> $a[] -> $b[]

///Build a new array whose elements are the results of applying the given function
///to each of the elements of the array.
val inline transform: $a[] -> ($a -> $b) -> $b[]

///Build a new array whose elements are the results of applying the given function
///to each of the elements of the array. The integer
///index passed to the funciton indicates the index of the element.
val inline transformi: $a[] -> (int -> $a -> $b) -> $b[]

///Build a .NET compatible array from the F# array.  Note that F# arrays and .NET
///compatible arrays are only different types when using .NET 1.0 or 1.1
val inline of_array: $a array -> $a[]

///Convert the given list to a .NET compatible array suitable for passing to .NET functions
val inline of_list: List< $a > -> $a[]

///Set the given element of the array
val inline set: $a[] -> int -> $a -> unit

///Build a new .NET compatible array that contains the given subrange specified by
///starting index and length 
val inline sub: $a[] -> int -> int -> $a[]

///Build an F# array from the given .NET compatible array.  Note that F# arrays and .NET
///compatible arrays are only different types when using .NET 1.0 or 1.1
val inline to_array: $a[] -> $a array

///Convert the given .NET compatible array to a list
val inline to_list: $a[] -> List< $a >

///Create a new array filled with zero (null or default) values.  Warning: Use of this
///function may result in subsequent exceptions if the elements of the array
///are accessed before initialized.
val inline zero_create: int -> $a[]

#endif
