

module Microsoft.FSharp.Primitives.Private
open Microsoft.FSharp.Primitives.Operators

type 'a ref = Microsoft.FSharp.Ref<'a>
type 'a option = Microsoft.FSharp.Option<'a>
type 'a list = Microsoft.FSharp.List<'a>

module CompatArray = Compatibility.CompatArray

let unbox x = Microsoft.FSharp.Primitives.Operators.op_Unbox x 
let box x = Microsoft.FSharp.Primitives.Operators.op_Box x

let (-) = Microsoft.FSharp.Primitives.Operators.op_IntegerSubtraction
let (+) = Microsoft.FSharp.Primitives.Operators.op_IntegerAddition
let ( * ) = Microsoft.FSharp.Primitives.Operators.op_IntegerMultiplication
let (/) = Microsoft.FSharp.Primitives.Operators.op_IntegerDivision
let (<) x y = Microsoft.FSharp.Primitives.Operators.op_StructuralLessThan x y
let (>) x y = Microsoft.FSharp.Primitives.Operators.op_StructuralGreaterThan x y
let (>=) x y = Microsoft.FSharp.Primitives.Operators.op_StructuralGreaterThanOrEqual x y
let (<=) x y = Primitives.Operators.op_StructuralLessThanOrEqual x y
let (:=) x y = Microsoft.FSharp.Primitives.Operators.op_Assign x y
let (.[]) x y = Microsoft.FSharp.Primitives.Operators.op_StringGet x y
let (!) x = Microsoft.FSharp.Primitives.Operators.op_Dereference x
let (=) x y = Microsoft.FSharp.Primitives.Operators.op_StructuralEquality x y
let (<>) x y = Microsoft.FSharp.Primitives.Operators.op_StructuralInequality x y
let (^) x y = Microsoft.FSharp.Primitives.Operators.op_StringConcatenation x y
let not x = Microsoft.FSharp.Primitives.Operators.op_BooleanNegation x
let raise x = Microsoft.FSharp.Primitives.Operators.op_Raise x
let ref x = Microsoft.FSharp.Primitives.Operators.op_CreateReference x
let max x y = Microsoft.FSharp.Primitives.Operators.op_StructuralMaximum x y
let min x y = Microsoft.FSharp.Primitives.Operators.op_StructuralMinimum x y
let (mod) = Microsoft.FSharp.Primitives.Operators.op_IntegerModulus
let (+.) (x:float) (y:float) =  Primitives.Operators.op_FloatAddition x y
let (-.) (x:float) (y:float) =  Primitives.Operators.op_FloatSubtraction x y
let ( *.) (x:float) (y:float) =  Primitives.Operators.op_FloatMultiplication x y
let (/.) (x:float) (y:float) =  Primitives.Operators.op_FloatDivision x y
let exp (x:float) = System.Math.Exp(x)
let log (x:float) = System.Math.Log(x)
let inline (lsl) (x) (y) = Primitives.Operators.op_OverloadedLeftShift x y

let failwith s = raise (new System.ArgumentException(s))

let code (c:char) = (# "" c : int)
let chr (n:int) =  (# "conv.u2" n : char)

let inline rethrow() = unbox(# "rethrow ldnull" : System.Object)
let inline (~-) x = Microsoft.FSharp.Primitives.Operators.op_OverloadedUnaryNegation x 

let byte_to_int (x:byte) = (# "" x : int)
let (land) (x) (y) = Primitives.Operators.op_OverloadedBitwiseAnd x y
let lnot (x) = Primitives.Operators.op_OverloadedLogicalNot x
let (.()) arr n = Primitives.Operators.op_ArrayGet arr n
let (.()<-) arr n x = Primitives.Operators.op_ArraySet arr n x
let bytearray_get (arr: byte[]) (n:int) =  (# "ldelem.i1" arr n : byte)  

module ArrayPrimitives = Microsoft.FSharp.Primitives.ArrayPrimitives

module List = struct
  let rec iter f x = match x with [] -> () | (h::t) -> f h; iter f t
  let rec filter p l = match l with [] -> [] | h::t -> if p h then h ::filter p t else filter p t
  let rec length l = match l with [] -> 0 | h::t -> 1 + length t
  let rec map f l = match l with [] -> [] | h::t -> f h :: map f t
  let rec find f l = match l with [] -> failwith "Not_found" | h::t -> if f h then h else find f t
end

let emptyArray arr = 
  Primitives.ArrayPrimitives.length arr = 0
let nonEmptyArray arr = 
  Primitives.ArrayPrimitives.length arr > 0

type 'a typ = { result: System.Type }

let inline typeof () : typ<'a> =
    let tok = (# "ldtoken !0" type('a) : System.RuntimeTypeHandle ) in
    let res = System.Type.GetTypeFromHandle(tok) in 
    { result = res }

let int_abs (x:int) = if x < 0 then -x else x
let string_of_int (x:int) = x.ToString()
let concat sep (strings : string list) =  
  System.String.Join(sep, CompatArray.of_list strings)
let primfloat (x:int) =  (# "conv.r8" x : float)
let int_of_char (c:char) = (# "" c : int)
let float_of_int (x:int) =  (# "conv.r8" x : float)

(*----------------------------------------------------------------------------
!* I32
 *--------------------------------------------------------------------------*)

module I32 = struct
  let add x y = x+y
  let sub x y = x-y
  let mul x y = x*y
  let div x y = x/y
  let rem x y = x mod y
  let int x   = (x:int)
  let lt  x y = x<(y:int)
  let rec pow x n =
    if n=0 then 1 else
      if n mod 2 = 0 then
	pow (x*x) (n / 2)
      else
	x* pow (x*x) (n / 2)
end


module I64 = struct
  type i64 = int64
  let add (x:int64) (y:int64) = (# "add" x y : int64 )
  let sub (x:int64) (y:int64) = (# "sub" x y : int64 )
  let mul (x:int64) (y:int64) = (# "mul" x y : int64 )
  let div (x:int64) (y:int64) = (# "div" x y : int64 )
  let rem (x:int64) (y:int64) = (# "rem" x y : int64 )
  let int (x:int)             = (# "conv.i8" x : int64)
  let lt  (x:int64) (y:int64) = (# "clt" x y : bool )
  let int_of (x:int64)        = (# "conv.i4" x : System.Int32)
  let rec pow x n =
    if n=0 then (int 1) else
      if n mod 2 = 0 then
	pow (mul x x) (n / 2)
      else
	mul x (pow (mul x x) (n / 2))
end

module R = struct (* for symmetry *)
  let add    x y = x +. y
  let sub    x y = x -. y
  let mul    x y = x *. y
  let div    x y = x /. y
  let negate x   = 0.0 -. x
  let powi   x n  = exp ( primfloat n *. log x)
  let equal  x (y:float) = x=y
  let lt     x (y:float) = x<y
  let gt     x (y:float) = x>y
  let lte    x (y:float) = x<=y
  let gte    x (y:float) = x>=y
  let negative x = x<0.0
  let positive x = x>0.0
  let to_float x = x
end

module Array = struct
  open Microsoft.FSharp.Primitives
  let length (arr: 'a array) =  ArrayPrimitives.length arr
  let get (arr: 'a array) (n:int) =  ArrayPrimitives.get arr n
  let set (arr: 'a array) (n:int) (x:'a) =  ArrayPrimitives.set arr n x
  let zero_create (n:int) = ArrayPrimitives.zero_create n
  let make  (n:int) (x:'a) =
    let arr = (zero_create n : 'a array) in 
    for i = 0 to n - 1 do 
      (set arr i x)
    done;
    arr
  let create (n:int) (x:'a) = make n x
  let init (n:int) (f: int -> 'a) = 
    let arr = (zero_create n : 'a array)  in 
    for i = 0 to n - 1 do 
      set arr i (f i)
    done;
    arr
  let map (f: 'a -> 'b) (arr:'a array) =
    let len = length arr in 
    let res = (zero_create len : 'b array) in 
    for i = 0 to len - 1 do 
      set res i (f (get arr i))
    done;
    res
  let sub (arr:'a array) (start:int) (len:int) =
   let res = (zero_create len : 'a array)  in
   for i = 0 to len - 1 do 
     (set res i (get arr (start + i) : 'a))
   done;
   res
  let inline sub_as_compat_array (arr:'a array) (start:int) (len:int) =
   let res = (CompatArray.zero_create len : 'a [])  in
   for i = 0 to len - 1 do 
     (CompatArray.set res i (get arr (start + i) : 'a))
   done;
   res
  let blit (arr1:'a array) (start1:int) (arr2: 'a array) (start2:int) (len:int) =
    for i = 0 to len - 1 do 
      (set arr2 (start2+i) (get arr1 (start1 + i) : 'a))
    done
  let copy (arr:'a array) =
    let len = length arr in 
    let res = (zero_create len : 'a array)  in
    blit arr 0 res 0 len;
    res
  let inline blit_from_compat_array (arr1:'a []) (start1:int) (arr2: 'a array) (start2:int) (len:int) =
    for i = 0 to len - 1 do 
      (set arr2 (start2+i) (CompatArray.get arr1 (start1 + i) : 'a))
    done
  let inline copy_from_compat_array (arr:'a[]) =
   let len = CompatArray.length arr in 
   let res = (zero_create len : 'a array)  in
   blit_from_compat_array arr 0 res 0 len;
   res
let fold_right (f : 'a -> 'b -> 'b) (arr:'a array) (acc: 'b) =
  let mutable res = acc in 
  let len = length arr in 
  for i = len - 1 downto 0 do 
    res <- f (get arr i) res
  done;
  res

  let to_list (arr:'a array) =
    let len = Primitives.ArrayPrimitives.length arr in 
    let mutable res = ([]: 'a list) in 
    for i = len - 1 downto 0 do 
      res <- (Primitives.ArrayPrimitives.get arr i) :: res
    done;
    res
end

let fst (a,b) = a
let snd (a,b) = b

module String = struct
  let concat sep (strings : string list) =  
    System.String.Join(sep, CompatArray.of_list strings)
  let inline get (str:string) i = str.get_Chars(i)
  let length (str:string) = str.Length
  let sub (s:string) (start:int) (len:int) =  
    try s.Substring(start,len)
    with :? System.ArgumentException -> failwith "String.sub" 
end

module Bytearray = struct

  type bytearray = byte[]
  let length (arr: byte[]) =  CompatArray.length arr
  let get (arr: byte[]) (n:int) =  (# "ldelem.i1" arr n : byte)  
  let set (arr: byte[]) (n:int) (x:byte) =  (# "stelem.i1" arr n x)  
  let zero_create (n:int) : byte[]= CompatArray.zero_create n
  let make  (n:int) = (zero_create n : byte[]) 

  let create (n:int)  = make n
  let init (n:int) (f: int -> byte) =  CompatArray.init n f
  let concat (arrs:byte[] list) = CompatArray.concat arrs
  let append arr1 arr2 = concat [arr1; arr2]
  let sub (arr:byte[]) (start:int) (len:int) = CompatArray.sub arr start len
  let fill (arr:byte[]) (start:int) (len:int) (x:byte) = CompatArray.fill arr start len x
  let copy (arr:byte[]) = CompatArray.copy arr
  let blit (arr1:byte[]) (start1:int) (arr2: byte[]) (start2:int) (len:int) = CompatArray.blit arr1 start1 arr2 start2 len
  let to_list (arr:byte[]) = CompatArray.to_list arr  
  let of_list (l:byte list) = CompatArray.of_list l
  let iter (f : byte -> unit) (arr:byte[]) = CompatArray.iter f arr
  let map (f: byte -> byte) (arr:byte[]) = CompatArray.map f arr
  let iteri (f : int -> byte -> unit) (arr:byte[]) = CompatArray.iteri f arr
  let mapi (f: int -> byte -> byte) (arr:byte[]) = CompatArray.mapi f arr
end

let EnumToInt x = (unbox (box x) : int)
let IntToEnum (x:int) = (unbox (box x) : 'a)

let rec combine_aux l  = 
  match l with
  | [] -> 0
  | h::t -> op_IntegerLogicalOr (EnumToInt h) (combine_aux t) 
let CombineEnumFlags (l : 'a list) = (IntToEnum (combine_aux l) : 'a)
let TestEnumFlag (x : 'a) (y : 'a) = op_StructuralInequality (op_IntegerLogicalAnd (EnumToInt x) (EnumToInt y)) 0

let (|>) x f = f x
let (<|) f x = f x

let rec foldR a b f z = if a<=b then foldR (a+1) b f (f z a) else z
let rec mapR a b f = if a<=b then f a :: mapR (a+1) b f else []

let forallR a b pred =
  let res = ref true in
  let i = ref a in
  while !res && !i <=b do
    if not (pred !i) then res := false;
    i := !i + 1
  done;
  !res

let firstR a b pred =
  let res = ref None in
  let i = ref a in
  while !res=None && !i <=b do
    if pred !i then res := Some !i;
    i := !i + 1
  done;
  !res

let rec upto a b = if a<= b then a::upto (a+1) b else []
let first pred xs =
  let rec ff = function
      [] -> None
    | x::xs -> if pred x then Some x else ff xs
  in
  ff xs

let ignore x = ()
