(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.Primitives.Operators

let op_Unbox (x:System.Object) = (# "unbox.any !0" type ('a) x : 'a)
let op_Box (x:'a) = (# "box !0" type ('a) x : System.Object)
let isinst (x:System.Object) = (op_Unbox (# "isinst !0" type ('a) x : System.Object) : 'a)
let nonnull (x:'a) = (# "ldnull cgt.un" (op_Box x) : bool)

let inline op_Raise (e:>System.Exception) = 
   match (e :> System.Object) with 
   | :? System.Exception -> (# "throw" (e :> System.Exception) : 'a)
   | _ ->  op_Unbox (# "ldnull"  : System.Object)

let op_CreateReference x = { contents = x }

let op_StructuralEquality (x : 'a) (y : 'a) = (# "ldc.i4 0 ceq" (Primitives.CompilerSpecific.StructuralCompareFast x y) : bool )
    when 'a = bool = (# "ceq" x y : bool )
    when 'a = int = (# "ceq" x y : bool )
    when 'a = sbyte = (# "ceq" x y : bool )
    when 'a = int16 = (# "ceq" x y : bool )
    when 'a = int32 = (# "ceq" x y : bool )
    when 'a = int64 = (# "ceq" x y : bool )
    when 'a = byte = (# "ceq" x y : bool )
    when 'a = uint16 = (# "ceq" x y : bool )
    when 'a = uint32 = (# "ceq" x y : bool )
    when 'a = uint64 = (# "ceq" x y : bool )
    when 'a = float = (# "ceq" x y : bool )
    when 'a = char = (# "ceq" x y : bool )
    when 'a = string = (# "call instance bool [mscorlib]System.String:: Equals(string)" x y : bool)

let op_IntegerNegation (x:int) = (# "neg" x : int) 
let (~-)(x:int) = (# "neg" x : int) 

let op_BooleanNegation (b:bool) = (# " ldc.i4 1 xor" b : bool)
let not (b:bool) = op_BooleanNegation b
let op_StructuralComparison (x : 'a) (y : 'a) = (Primitives.CompilerSpecific.StructuralCompareFast x y)
   when 'a = bool = (# "sub" x y : int )
   when 'a = sbyte = (# "sub" x y : int )
   when 'a = int16 = (# "sub" x y : int )
   when 'a = int32 = if (# "clt" x y : bool ) then (-1) else (# "cgt" x y : int )
   when 'a = int = if (# "clt" x y : bool ) then (-1) else (# "cgt" x y : int )
   when 'a = int64 = if (# "clt" x y : bool ) then (-1) else (# "cgt" x y : int )
   when 'a = byte = (# "sub" x y : int )
   when 'a = uint16 = (# "sub" x y : int )
   when 'a = uint32 = if (# "clt.un" x y : bool ) then (-1) else (# "cgt.un" x y : int )
   when 'a = uint64 = if (# "clt.un" x y : bool ) then (-1) else (# "cgt.un" x y : int )
   when 'a = float = if (# "clt" x y : bool ) then (-1) else (# "cgt" x y : int )
   when 'a = char = (# "sub" x y : int )
   when 'a = string = (# "call int32 [mscorlib]System.String:: CompareOrdinal(string,string)" x y : int)
   
let op_StructuralInequality (x:'a) (y:'a) = (# "ldc.i4 1 xor" (op_StructuralEquality x y) : bool)
    when 'a = bool = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = int = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = sbyte = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = int16 = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = int32 = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = int64 = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = byte = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = uint16 = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = uint32 = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = uint64 = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = float = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = char = (# "ceq ldc.i4 1 xor" x y : bool )
    when 'a = string = (# "call instance bool [mscorlib]System.String:: Equals(string) ldc.i4 1 xor" x y : bool)
    
let op_StructuralLessThan (x:'a) (y:'a) = (# "ldc.i4 0 clt" (op_StructuralComparison x y) : bool)
    when 'a = bool = (# "clt" x y : bool )
    when 'a = int = (# "clt" x y : bool )
    when 'a = sbyte = (# "clt" x y : bool )
    when 'a = int16 = (# "clt" x y : bool )
    when 'a = int32 = (# "clt" x y : bool )
    when 'a = int64 = (# "clt" x y : bool )
    when 'a = byte = (# "clt.un" x y : bool )
    when 'a = uint16 = (# "clt.un" x y : bool )
    when 'a = uint32 = (# "clt.un" x y : bool )
    when 'a = uint64 = (# "clt.un" x y : bool )
    when 'a = float = (# "clt" x y : bool )
    when 'a = char = (# "clt" x y : bool )
    
let op_StructuralGreaterThan (x:'a) (y:'a) = (# "cgt" (op_StructuralComparison x y) 0 : bool)
    when 'a = bool = (# "cgt" x y : bool )
    when 'a = int = (# "cgt" x y : bool )
    when 'a = sbyte = (# "cgt" x y : bool )
    when 'a = int16 = (# "cgt" x y : bool )
    when 'a = int32 = (# "cgt" x y : bool )
    when 'a = int64 = (# "cgt" x y : bool )
    when 'a = byte = (# "cgt.un" x y : bool )
    when 'a = uint16 = (# "cgt.un" x y : bool )
    when 'a = uint32 = (# "cgt.un" x y : bool )
    when 'a = uint64 = (# "cgt.un" x y : bool )
    when 'a = float = (# "cgt" x y : bool )
    when 'a = char = (# "cgt" x y : bool )
let op_StructuralLessThanOrEqual (x:'a) (y:'a) = not (op_StructuralGreaterThan x y)
    when 'a = bool = (# "cgt ldc.i4 1 xor" x y : bool )
    when 'a = int = (# "cgt ldc.i4 1 xor" x y : bool )
    when 'a = sbyte = (# "cgt ldc.i4 1 xor" x y : bool )
    when 'a = int16 = (# "cgt ldc.i4 1 xor" x y : bool )
    when 'a = int32 = (# "cgt ldc.i4 1 xor" x y : bool )
    when 'a = int64 = (# "cgt ldc.i4 1 xor" x y : bool )
    when 'a = byte = (# "cgt.un ldc.i4 1 xor" x y : bool )
    when 'a = uint16 = (# "cgt.un ldc.i4 1 xor" x y : bool )
    when 'a = uint32 = (# "cgt.un ldc.i4 1 xor" x y : bool )
    when 'a = uint64 = (# "cgt.un ldc.i4 1 xor" x y : bool )
    when 'a = float = (# "cgt ldc.i4 1 xor" x y : bool )
    when 'a = char = (# "cgt ldc.i4 1 xor" x y : bool )

let op_StructuralGreaterThanOrEqual (x:'a) (y:'a) = not (op_StructuralLessThan x y)
    when 'a = bool = (# "clt ldc.i4 1 xor" x y : bool )
    when 'a = int = (# "clt ldc.i4 1 xor" x y : bool )
    when 'a = sbyte = (# "clt ldc.i4 1 xor" x y : bool )
    when 'a = int16 = (# "clt ldc.i4 1 xor" x y : bool )
    when 'a = int32 = (# "clt ldc.i4 1 xor" x y : bool )
    when 'a = int64 = (# "clt ldc.i4 1 xor" x y : bool )
    when 'a = byte = (# "clt.un ldc.i4 1 xor" x y : bool )
    when 'a = uint16 = (# "clt.un ldc.i4 1 xor" x y : bool )
    when 'a = uint32 = (# "clt.un ldc.i4 1 xor" x y : bool )
    when 'a = uint64 = (# "clt.un ldc.i4 1 xor" x y : bool )
    when 'a = float = (# "clt ldc.i4 1 xor" x y : bool )
    when 'a = char = (# "clt ldc.i4 1 xor" x y : bool )

let op_StructuralMinimum x y = if op_StructuralLessThan x y then x else y
let op_StructuralMaximum x y = if op_StructuralLessThan x y then y else x
let op_GenericEq (x: 'a)  (y: 'a) = Primitives.CompilerSpecific.GenericEq x y
    when 'a = bool = (# "ceq" x y : bool )
    when 'a = int = (# "ceq" x y : bool )
    when 'a = sbyte = (# "ceq" x y : bool )
    when 'a = int16 = (# "ceq" x y : bool )
    when 'a = int32 = (# "ceq" x y : bool )
    when 'a = int64 = (# "ceq" x y : bool )
    when 'a = byte = (# "ceq" x y : bool )
    when 'a = uint16 = (# "ceq" x y : bool )
    when 'a = uint32 = (# "ceq" x y : bool )
    when 'a = uint64 = (# "ceq" x y : bool )
    when 'a = float = (# "ceq" x y : bool )
    when 'a = char = (# "ceq" x y : bool )
    when 'a = string = (# "ceq" x y : bool )
let op_GenericNotEq (x:'a) (y:'a) = not (op_GenericEq x y)

let op_StructuralHash (x: 'a) = (Primitives.CompilerSpecific.StructuralHash x)
          (* WARNING: any type specializations must return identical values to *)
          (* the ::GetHashCode methods on the given types. (We could inline directly from *)
          (* the appropriate version of mscorlib here...) *)
        (* Note: int16, uint16, sbyte etc. have funky hash codes *)
    when 'a = bool = (# "" x : int )
    when 'a = int = (# "" x : int )
    when 'a = int32 = (# "" x : int )
    when 'a = byte = (# "" x : int )
    when 'a = uint32 = (# "" x : int )
    when 'a = string = (# "call instance int32 [mscorlib]System.String:: GetHashCode()" x : int)


let op_IntegerAddition (x:int) (y:int) = (# "add" x y : int)
let op_IntegerSubtraction (x:int) (y:int) = (# "sub" x y : int)
let op_IntegerMultiplication (x:int) (y:int) = (# "mul" x y : int)
let op_IntegerDivision (x:int) (y:int) = (# "div" x y : int)
let op_IntegerModulus (x:int) (y:int) = (# "rem" x y : int)
let op_IntegerLogicalShiftLeft (x:int) (y:int) = (# "shl" x y : int)
let op_IntegerLogicalShiftRight (x:int) (y:int) = (# "shr.un" x y : int)
let op_IntegerArithmeticShiftRight (x:int) (y:int) = (# "shr" x y : int)

let op_FloatNegation (x:float) =  (# "neg" x : float)
let op_FloatAddition (x:float) (y:float) =  (# "add" x y : float)
let op_FloatSubtraction (x:float) (y:float) =  (# "sub" x y : float)
let op_FloatMultiplication (x1:float) (y1:float) =  (# "mul" x1 y1 : float)
let op_FloatDivision (x:float) (y:float) =  (# "div" x y : float)
let op_FloatExponentiation (x:float) (y:float) =  System.Math.Pow(x,y)

let op_StringConcatenation (x:string) (y:string) = System.String.Concat(x,y)
let op_Dereference x = x.contents
let op_Assign x y = x.contents <- y

let nonempty x = match x with [] -> false | _ -> true

(* let rec (@) x y = match x with [] -> y | (h::t) -> h :: (t @ y)*)
let op_ListAppend l1 l2 = 
  match l1 with
  | [] -> l2
  | (h::t) -> 
  match l2 with
  | [] -> l1
  | _ -> 
      let res = [h] in 
      let mutable curr = t in 
      let mutable cell = res in 
      while nonempty curr do
	let nw = [curr.(::).0] in 
	cell.(::).1 <- nw;
	curr <- curr.(::).1;
	cell <- nw
      done;
      cell.(::).1 <- l2;
      res

let op_ArrayGet (arr: 'a array) (n:int) =  (# "ldelem.erasable !0" type ('a) arr n : 'a)  
let op_ArraySet (arr: 'a array) (n:int) (x:'a) =  (# "stelem.erasable !0" type ('a) arr n x)  
let op_StringGet (s:string) (n:int) = s.Chars(n)

let op_IntegerLogicalOr (x:int) (y:int) = (# "or" x y : int)
let op_IntegerLogicalXor (a:int) (b:int) = (# "xor" a b : int)
let op_IntegerLogicalAnd (a:int) (b:int) = (# "and" a b : int)
let op_IntegerLogicalNot (x:int) = (# "not" x : int)

let inline op_OverloadedAddition (x: ^a) (y: ^b) : ^a = 
   (^a.op_Addition(x,y))
   when ^a = int32  and ^b = int32 = (# "add" x y : int32 )
   when ^a = float  and ^b = float = (# "add" x y : float )
   when ^a = float32  and ^b = float32 = (# "add" x y : float32 )
   when ^a = int64  and ^b = int64 = (# "add" x y : int64 )
   when ^a = uint64  and ^b = uint64 = (# "add" x y : uint64 )
   when ^a = uint32  and ^b = uint32 = (# "add" x y : uint32 )
   when ^a = int16  and ^b = int16 = (# "add" x y : int16 )
   when ^a = uint16  and ^b = uint16 = (# "add" x y : uint16 )
   when ^a = nativeint  and ^b = nativeint = (# "add" x y : nativeint )
   when ^a = unativeint  and ^b = unativeint = (# "add" x y : unativeint )
   when ^a = sbyte  and ^b = sbyte = (# "add" x y : sbyte )
   when ^a = byte  and ^b = byte = (# "add" x y : byte )
   when ^a = string  and ^b = string = System.String.Concat(((x :> obj) :?> string),((y :> obj) :?> string))

let inline op_OverloadedSubtraction (x: ^a) (y: ^b) : ^a = 
   (^a.op_Subtraction(x,y))
   when ^a = int32  and ^b = int32 = (# "sub" x y : int32 )
   when ^a = float  and ^b = float = (# "sub" x y : float )
   when ^a = float32  and ^b = float32 = (# "sub" x y : float32 )
   when ^a = int64  and ^b = int64 = (# "sub" x y : int64 )
   when ^a = uint64  and ^b = uint64 = (# "sub" x y : uint64 )
   when ^a = uint32  and ^b = uint32 = (# "sub" x y : uint32 )
   when ^a = int16  and ^b = int16 = (# "sub" x y : int16 )
   when ^a = uint16  and ^b = uint16 = (# "sub" x y : uint16 )
   when ^a = nativeint  and ^b = nativeint = (# "sub" x y : nativeint )
   when ^a = unativeint  and ^b = unativeint = (# "sub" x y : unativeint )
   when ^a = sbyte  and ^b = sbyte = (# "sub" x y : sbyte )
   when ^a = byte  and ^b = byte = (# "sub" x y : byte )


let inline op_OverloadedMultiplication (x: ^a) (y: ^b) : ^a = 
   (^a.op_Multiply(x,y))
   when ^a = int32  and ^b = int32 = (# "mul" x y : int32 )
   when ^a = float  and ^b = float = (# "mul" x y : float )
   when ^a = float32  and ^b = float32 = (# "mul" x y : float32 )
   when ^a = int64  and ^b = int64 = (# "mul" x y : int64 )
   when ^a = uint64  and ^b = uint64 = (# "mul" x y : uint64 )
   when ^a = uint32  and ^b = uint32 = (# "mul" x y : uint32 )
   when ^a = int16  and ^b = int16 = (# "mul" x y : int16 )
   when ^a = uint16  and ^b = uint16 = (# "mul" x y : uint16 )
   when ^a = nativeint  and ^b = nativeint = (# "mul" x y : nativeint )
   when ^a = unativeint  and ^b = unativeint = (# "mul" x y : unativeint )
   when ^a = sbyte  and ^b = sbyte = (# "mul" x y : sbyte )
   when ^a = byte  and ^b = byte = (# "mul" x y : byte )

let inline op_OverloadedDivision (x: ^a) (y: ^b) : ^a = 
   (^a.op_Division(x,y))
   when ^a = int32  and ^b = int32 = (# "div" x y : int32 )
   when ^a = float  and ^b = float = (# "div" x y : float )
   when ^a = float32  and ^b = float32 = (# "div" x y : float32 )
   when ^a = int64  and ^b = int64 = (# "div" x y : int64 )
   when ^a = uint64  and ^b = uint64 = (# "div.un" x y : uint64 )
   when ^a = uint32  and ^b = uint32 = (# "div.un" x y : uint32 )
   when ^a = int16  and ^b = int16 = (# "div" x y : int16 )
   when ^a = uint16  and ^b = uint16 = (# "div.un" x y : uint16 )
   when ^a = nativeint  and ^b = nativeint = (# "div" x y : nativeint )
   when ^a = unativeint  and ^b = unativeint = (# "div.un" x y : unativeint )
   when ^a = sbyte  and ^b = sbyte = (# "div" x y : sbyte )
   when ^a = byte  and ^b = byte = (# "div.un" x y : byte )

let inline op_OverloadedModulus (x: ^a) (y: ^b) : ^a = 
   (^a.op_Modulus(x,y))
   when ^a = int32  and ^b = int32 = (# "rem" x y : int32 )
   when ^a = float  and ^b = float = (# "rem" x y : float )
   when ^a = float32  and ^b = float32 = (# "rem" x y : float32 )
   when ^a = int64  and ^b = int64 = (# "rem" x y : int64 )
   when ^a = uint64  and ^b = uint64 = (# "rem.un" x y : uint64 )
   when ^a = uint32  and ^b = uint32 = (# "rem.un" x y : uint32 )
   when ^a = int16  and ^b = int16 = (# "rem" x y : int16 )
   when ^a = uint16  and ^b = uint16 = (# "rem.un" x y : uint16 )
   when ^a = nativeint  and ^b = nativeint = (# "rem" x y : nativeint )
   when ^a = unativeint  and ^b = unativeint = (# "rem.un" x y : unativeint )
   when ^a = sbyte  and ^b = sbyte = (# "rem" x y : sbyte )
   when ^a = byte  and ^b = byte = (# "rem.un" x y : byte )

let inline op_OverloadedUnaryNegation (x: ^a) : ^a = 
   (^a.op_UnaryNegation(x))
   when ^a = int32   = (# "neg" x  : int32 )
   when ^a = float   = (# "neg" x  : float )
   when ^a = float32  = (# "neg" x  : float32 )
   when ^a = int64   = (# "neg" x  : int64 )
   when ^a = int16   = (# "neg" x  : int16 )
   when ^a = nativeint = (# "neg" x  : nativeint )
   when ^a = sbyte   = (# "neg" x  : sbyte )

let inline op_OverloadedUnaryPlus (x: ^a) : ^a =
   (^a.op_UnaryPlus(x))
   when ^a = int32   = x
   when ^a = float   = x
   when ^a = float32  = x
   when ^a = int64   = x
   when ^a = uint64  = x
   when ^a = uint32  = x
   when ^a = int16   = x
   when ^a = uint16  = x
   when ^a = nativeint = x
   when ^a = unativeint = x
   when ^a = sbyte   = x
   when ^a = byte  = x

let inline op_OverloadedLeftShift (x: ^a) (n:int) : ^a = 
   (^a.op_LeftShift(x,n))
   when ^a = int32   = (# "shl" x n : int32 )
   when ^a = int64   = (# "shl" x n : int64 )
   when ^a = uint64  = (# "shl" x n : uint64 )
   when ^a = uint32  = (# "shl" x n : uint32 )
   when ^a = int16   = (# "shl" x n : int16 )
   when ^a = uint16  = (# "shl" x n : uint16 )
   when ^a = nativeint = (# "shl" x n : nativeint )
   when ^a = unativeint = (# "shl" x n : unativeint )
   when ^a = sbyte   = (# "shl" x n : sbyte )
   when ^a = byte  = (# "shl" x n : byte )

let inline op_OverloadedRightShift (x: ^a) (n:int) : ^a = 
   (^a.op_RightShift(x,n))
   when ^a = int32   = (# "shr" x n : int32 )
   when ^a = int64   = (# "shr" x n : int64 )
   when ^a = uint64  = (# "shr.un" x n : uint64 )
   when ^a = uint32  = (# "shr.un" x n : uint32 )
   when ^a = int16   = (# "shr" x n : int16 )
   when ^a = uint16  = (# "shr.un" x n : uint16 )
   when ^a = nativeint = (# "shr" x n : nativeint )
   when ^a = unativeint = (# "shr.un" x n : unativeint )
   when ^a = sbyte   = (# "shr" x n : sbyte )
   when ^a = byte  = (# "shr.un" x n : byte )

let inline op_OverloadedLogicalRightShift (x: ^a) (n:int) : ^a = 
   (^a.op_RightShift(x,n))
   when ^a = int32   = (# "shr.un" x n : int32 )
   when ^a = int64   = (# "shr.un" x n : int64 )
   when ^a = uint64  = (# "shr.un" x n : uint64 )
   when ^a = uint32  = (# "shr.un" x n : uint32 )
   when ^a = int16   = (# "shr.un" x n : int16 )
   when ^a = uint16  = (# "shr.un" x n : uint16 )
   when ^a = nativeint = (# "shr.un" x n : nativeint )
   when ^a = unativeint = (# "shr.un" x n : unativeint )
   when ^a = sbyte   = (# "shr.un" x n : sbyte )
   when ^a = byte  = (# "shr.un" x n : byte )

let inline op_OverloadedBitwiseAnd (x: ^a) (y: ^a) : ^a = 
   (^a.op_BitwiseAnd(x,y))
   when ^a = int32   = (# "and" x y : int32 )
   when ^a = int64   = (# "and" x y : int64 )
   when ^a = uint64  = (# "and" x y : uint64 )
   when ^a = uint32  = (# "and" x y : uint32 )
   when ^a = int16   = (# "and" x y : int16 )
   when ^a = uint16  = (# "and" x y : uint16 )
   when ^a = nativeint = (# "and" x y : nativeint )
   when ^a = unativeint = (# "and" x y : unativeint )
   when ^a = sbyte   = (# "and" x y : sbyte )
   when ^a = byte  = (# "and" x y : byte )

let inline op_OverloadedBitwiseOr (x: ^a) (y: ^a) : ^a = 
   (^a.op_BitwiseOr(x,y))
   when ^a = int32   = (# "or" x y : int32 )
   when ^a = int64   = (# "or" x y : int64 )
   when ^a = uint64  = (# "or" x y : uint64 )
   when ^a = uint32  = (# "or" x y : uint32 )
   when ^a = int16   = (# "or" x y : int16 )
   when ^a = uint16  = (# "or" x y : uint16 )
   when ^a = nativeint = (# "or" x y : nativeint )
   when ^a = unativeint = (# "or" x y : unativeint )
   when ^a = sbyte   = (# "or" x y : sbyte )
   when ^a = byte  = (# "or" x y : byte )

let inline op_OverloadedExclusiveOr (x: ^a) (y: ^a) : ^a = 
   (^a.op_ExclusiveOr(x,y))
   when ^a = int32   = (# "xor" x y : int32 )
   when ^a = int64   = (# "xor" x y : int64 )
   when ^a = uint64  = (# "xor" x y : uint64 )
   when ^a = uint32  = (# "xor" x y : uint32 )
   when ^a = int16   = (# "xor" x y : int16 )
   when ^a = uint16  = (# "xor" x y : uint16 )
   when ^a = nativeint = (# "xor" x y : nativeint )
   when ^a = unativeint = (# "xor" x y : unativeint )
   when ^a = sbyte   = (# "xor" x y : sbyte )
   when ^a = byte  = (# "xor" x y : byte )

let inline op_OverloadedLogicalNot (x: ^a) : ^a = 
   (^a.op_LogicalNot(x))
   when ^a = int32   = (# "not" x : int32 )
   when ^a = int64   = (# "not" x : int64 )
   when ^a = uint64  = (# "not" x : uint64 )
   when ^a = uint32  = (# "not" x : uint32 )
   when ^a = int16   = (# "not" x : int16 )
   when ^a = uint16  = (# "not" x : uint16 )
   when ^a = nativeint = (# "not" x : nativeint )
   when ^a = unativeint = (# "not" x : unativeint )
   when ^a = sbyte   = (# "not" x : sbyte )
   when ^a = byte  = (# "not" x : byte )


