(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.Arg

type spec =
  | Clear of (bool ref)       
  | Float of (float -> unit)
  | Int of (int -> unit)    
  | Rest of (string -> unit)
  | Set of (bool ref)
  | String of (string -> unit)
  | Unit of (unit -> unit)  

let get_usage specs u =  
  let sbuf = Buffer.create 100 in 
  let pstring s = Buffer.add_string sbuf s in
  let pendline s = pstring s; Buffer.add_string sbuf "\n" in
  pendline u;
  List.iter (function 
    | (s, (Unit _ | Set _ | Clear _), x) -> pstring "\t"; pstring s; pstring ": "; pendline x
    | (s, String f, x) -> pstring "\t"; pstring s; pstring " <string>: "; pendline x
    | (s, Int f, x) -> pstring "\t"; pstring s; pstring " <int>: "; pendline x
    | (s, Float f, x) ->  pstring "\t"; pstring s; pstring " <float>: "; pendline x
    | (s, Rest f, x) -> pstring "\t"; pstring s; pstring " ...: "; pendline x)
    specs;
  pstring "\t"; pstring "--help"; pstring ": "; pendline "display this list of options";
  pstring "\t"; pstring "-help"; pstring ": "; pendline "display this list of options";
  Buffer.contents sbuf

let usage specs u = prerr_endline (get_usage specs u)
type argspec = (string * spec * string) 

exception Help of string
exception Bad of string

let parse_argv cursor argv specs other usageText =
  let nargs = Array.length argv in 
  incr cursor;
  while !cursor < nargs do
    let arg = argv.(!cursor) in 
    let rec findMatchingArg (l: argspec  list) = 
      match l with 
      | ((s, action, _) :: _) when s = arg -> 
         let getSecondArg () = 
           if !cursor + 1 >= nargs then 
             raise(Bad("option "^s^" needs an argument.\n"^get_usage specs usageText));
           argv.(!cursor+1) in
           
         begin match action with 
         | Unit f -> 
           f (); 
           incr cursor
         | Set f ->
           f := true; 
           incr cursor
         | Clear f -> 
           f := false; 
           incr cursor
         | String f-> 
           let arg2 = getSecondArg() in 
           f arg2; 
           cursor := !cursor + 2
         | Int f -> 
           let arg2 = getSecondArg () in 
           let arg2 = try int_of_string arg2 with _ -> raise(Bad(get_usage specs usageText)); exit 1 in  
           f arg2;
           cursor := !cursor + 2;
         | Float f -> 
           let arg2 = getSecondArg() in 
           let arg2 = try float_of_string arg2 with _ -> raise(Bad(get_usage specs usageText)); exit 1 in 
           f arg2; 
           cursor := !cursor + 2;
         | Rest f -> 
           incr cursor;
           while !cursor < nargs do
             f (argv.(!cursor));
             incr cursor;
           done;
         end
      | (_ :: more)  -> findMatchingArg more 
      | [] -> 
          if arg = "-help" or arg = "--help" then begin
            raise (Help (get_usage specs usageText))
          end else if String.get arg 0 = '-' then begin
            raise (Bad ("unrecognized argument: "^ arg ^ "\n" ^ get_usage specs usageText)); 
          end else begin
             other arg;
             incr cursor
          end in 
    findMatchingArg specs 
  done

let current = ref 0
let parse specs other usage = 
  try parse_argv current Sys.argv specs other usage
  with Bad h |Help h -> prerr_endline h; flush(stderr); exit 1

