(*==========================================================================
 * Args: ML-style Command-line argument processing
 *
 * This module is only included to make it possible to cross-compile 
 * code with other ML compilers.  It may be deprecated and/or removed in 
 * a future release. You may wish to use .NET functions directly instead. 
 *
 * (c) Microsoft Corporation 2004.  All code and documentation in this file 
 * has been written 'from scratch'.  The interface to the module 
 * is similar to that found in versions of other ML implementations, 
 * but is not an exact match.  The type signatures in this interface
 * are an edited version of those generated automatically by running 
 * "bin\fsc.exe -i" on the implementation file.
 *=========================================================================*)

module Microsoft.FSharp.MLLib.Arg

(* --------------------------------------------------------------------
 * "parse specs f use" parses the arguments given by Sys.argv
 * according to the argument processing specifications "specs".
 * Arguments begin with "-". Non-arguments are passed to "f" in
 * order.  "use" is printed as part of the usage line if an error occurs.
 *
 * Permitted arguments are specified using triples: (arg, action, help).
 * Actions are:
 *   Unit(f): call f, no subseq. arg
 *   Set(br): set ref to 'true', no subseq. arg.
 *   Clear(br): set ref to 'false, no subseq. arg.
 *   String(f): pass the subseq. arg to f
 *   Int(f): pass the subseq. arg to f
 *   Float(f): pass the subseq. arg to f
 *   Rest(f): pass all subseq. args to f in order
 *
 * "usage specs use" prints the help for each argument.
 * -------------------------------------------------------------------- *)
 
type argspec = (string * spec * string)
and spec = 
  | Clear of bool ref
  | Float of (float -> unit)
  | Int of (int -> unit)
  | Rest of (string -> unit)
  | Set of bool ref
  | String of (string -> unit)
  | Unit of (unit -> unit)
val parse: argspec list -> (string -> unit) -> string -> unit
val parse_argv: int ref -> string array -> argspec list -> (string -> unit) -> string -> unit
val usage: argspec list -> string -> unit
exception Bad of string
exception Help of string
