(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.Filename

open System.IO

let current_dir_name = "."
let parent_dir_name =  ".."
let concat (x:string) (y:string) = Path.Combine (x,y)
let is_relative (s:string) = not (Path.IsPathRooted(s))

let check_suffix (x:string) (y:string) = x.EndsWith(y)

let chop_suffix (x:string) (y:string) =
  if not (check_suffix x y) then invalid_arg "chop_suffix";
  String.sub x 0 (String.length x - String.length y)

let has_extension (s:string) = 
  (String.length s >= 1 && String.get s (String.length s - 1) = '.') 
  or Path.HasExtension(s)

let chop_extension (s:string) =
  if not (has_extension s) then invalid_arg "chop_extension";
  Path.Combine (Path.GetDirectoryName s,Path.GetFileNameWithoutExtension(s))


let basename (s:string) = 
  Path.GetFileName(s)
let dirname (s:string) = 
  if s = "" then "."
  else 
    match Path.GetDirectoryName(s) with 
    | null -> if Path.IsPathRooted(s) then s else "."
    | res -> if res = "" then "." else res

let is_implicit (s:string) = 
  is_relative s &
  let d = dirname s in 
  d <> current_dir_name & d <> parent_dir_name


let temp_file (p:string) (s:string) = Path.GetTempFileName()

let quote s = "\'" ^ s ^ "\'"

