(*==========================================================================
 * Filename: ML-like filename and path manipulation operations.
 *
 * This module is only included to make it possible to cross-compile 
 * code with other ML compilers.  It may be deprecated and/or removed in 
 * a future release. 
 * 
 * (c) Microsoft Corporation 2004.  All code and documentation in this file 
 * has been written 'from scratch'.  The interface to the module 
 * is similar to that found in versions of other ML implementations, 
 * but is not an exact match.  The type signatures in this interface
 * are an edited version of those generated automatically by running 
 * "bin\fsc.exe -i" on the implementation file.
 *=========================================================================*)

module Microsoft.FSharp.MLLib.Filename

(*--------------------------------------------------------------------------
 * "check_suffix f s" returns true if filename "f" ends in suffix "s",
 * e.g. check_suffix "abc.ml" ".ml" returns true.
 * Assuming "check_suffix f s" holds, "chop_suffix f s" returns the
 * filename "f" with the suffix "s" removed.
 *
 * "chop_extension f" removes the extension, if any, from the given
 * filename.
 *--------------------------------------------------------------------------*)
 
val check_suffix: string -> string -> bool
val chop_extension: string -> string
val chop_suffix: string -> string -> string

(*--------------------------------------------------------------------------
 * "concat a b" returns System.IO.Path.Combine(a,b), i.e. the
 * two names conjoined by the appropriate directory separator character
 * for this architecture.
 *
 * "dirname" and "basename" decompose a filename into a directory name
 * and a filename, i.e. "concat (dirname s) (basename s) = s"
 *--------------------------------------------------------------------------*)
 
val concat: string -> string -> string
val dirname: string -> string
val basename: string -> string

(*--------------------------------------------------------------------------
 * "current_dir_name" returns ".", i.e. the name used for the current
 * directory on this OS. "parent_dir_name" returns "..",
 * i.e. the name for the directory above the current directory on
 * this OS.
 *--------------------------------------------------------------------------*)

val current_dir_name: string
val parent_dir_name: string

val has_extension: string -> bool

(*--------------------------------------------------------------------------
 * "is_relative s" returns "not (System.IO.Path.IsPathRooted(s))",
 * i.e. indicates if the path is relative to the current directory or
 * absolute.
 *
 * "is_implicit s" returns "is_relative s & (dirname s) <> current_dir_name &
 * (dirname s <> parent_dir_name)"
 *--------------------------------------------------------------------------*)
 
val is_implicit: string -> bool
val is_relative: string -> bool

(*--------------------------------------------------------------------------
 * "quote s" is designed for use to quote a filename when using it
 * for a system command.  It returns ("\'" ^ s ^ "\'").  
 *--------------------------------------------------------------------------*)
 
val quote: string -> string

(*--------------------------------------------------------------------------
 * "temp_file f s" returns a previously unused new file name.  "f" and "s"
 * are hints as to a suitable file name and suffix for the file.
 *--------------------------------------------------------------------------*)
 
val temp_file: string -> string -> string
