(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.Int32

let compare (x:int32) y = Pervasives.compare x y

let zero = 0
let one = 1
let minus_one = (-1)
let neg (x:int32) =  -x
let add (x:int32) (y:int32) = x + y
let sub (x:int32) (y:int32) = x - y
let mul (x:int32) (y:int32) = x * y
let div (x:int32) (y:int32) = x / y
let rem (x:int32) (y:int32) = x mod y
let succ (x:int32) = x + 1
let pred (x:int32) = x - 1
let abs (x:int32) = if x < zero then neg x else x
let max_int = Pervasives.max_int
let min_int = Pervasives.min_int
let logand (x:int32) (y:int32) = x land y
let logor (x:int32) (y:int32) = x lor y
let logxor (x:int32) (y:int32) = x lxor y
let lognot (x:int32) = lnot x
let shift_left (x:int32) (n:int) =  x lsl n
let shift_right (x:int32) (n:int) =  x asr n
let shift_right_logical (x:int32) (n:int) =  x lsr n
let of_int (n:int) =  n
let to_int (x:int32) = x
let of_float (f:float) =  (# "conv.i4" f : int32)
let to_float (x:int32) =  (# "conv.r8" x : float)

let of_float32 (f:float32) =  (# "conv.i4" f : int32)
let to_float32 (x:int32) =  (# "conv.r4" x : float32)

let of_uint32 (f:uint32) =  (# "conv.i4" f : int32)
let to_uint32 (x:int32) =  (# "conv.u4" x : uint32)

let of_int64 (f:int64) =  (# "conv.i4" f : int32)
let to_int64 (x:int32) =  (# "conv.i8" x : int64)

let of_nativeint (f:nativeint) =  (# "conv.i4" f : int32)
let to_nativeint (x:int32) =  (# "conv.u" x : nativeint)

let of_string (s:string) = int_of_string s
let to_string (x:int32) = (box x).ToString()

(* Yes folks, the only verifiable way to get the bits from a float *)
(* is to blast it to a stream then read it back in... *)
let bits_of_float32 (x:float32) = 
   let ms = new System.IO.MemoryStream() in 
   let bw = new System.IO.BinaryWriter(ms) in 
   bw.Write(x); 
   ms.Flush();
   let br = new System.IO.BinaryReader(ms) in 
   ignore(ms.Seek(0L, System.IO.SeekOrigin.Begin)); 
   let res = br.ReadInt32() in 
   br.Close(); 
   res

let float32_of_bits (x:int32) = 
   let ms = new System.IO.MemoryStream() in 
   let bw = new System.IO.BinaryWriter(ms) in 
   bw.Write(x); 
   ms.Flush();
   let br = new System.IO.BinaryReader(ms) in 
   ignore(ms.Seek(0L, System.IO.SeekOrigin.Begin)); 
   let res = br.ReadSingle() in 
   br.Close(); 
   res


let float32_to_float (x:float32) = (# "conv.r8" x : float)
let float_to_float32 (x:float) = (# "conv.r4" x : float32)

let float_of_bits x = float32_to_float (float32_of_bits x)
let bits_of_float x = bits_of_float32 (float_to_float32 x)
