(*==========================================================================
 * Int32: Basic operations for ML compatibility
 *
 * The type int32 is identical to int. 
 *
 * (c) Microsoft Corporation 2004.  All code and documentation in this file 
 * has been written 'from scratch'.  The interface to the module 
 * is similar to that found in versions of other ML implementations, 
 * but is not an exact match.  The type signatures in this interface
 * are an edited version of those generated automatically by running 
 * "bin\fsc.exe -i" on the implementation file.
 *=========================================================================*)


module Microsoft.FSharp.MLLib.Int32

val zero: int32
val one: int32
val minus_one: int32
val pred: int32 -> int32
val max_int: int32
val min_int: int32
val succ: int32 -> int32

val abs: int32 -> int32
val add: int32 -> int32 -> int32
val div: int32 -> int32 -> int32
val mul: int32 -> int32 -> int32
val neg: int32 -> int32
val rem: int32 -> int32 -> int32
val sub: int32 -> int32 -> int32

val compare: int32 -> int32 -> int

val logand: int32 -> int32 -> int32
val lognot: int32 -> int32
val logor: int32 -> int32 -> int32
val logxor: int32 -> int32 -> int32
val shift_left: int32 -> int -> int32
val shift_right: int32 -> int -> int32
val shift_right_logical: int32 -> int -> int32

val of_float: float -> int32
val to_float: int32 -> float

val of_float32: float32 -> int32
val to_float32: int32 -> float32

(* In F#, type int32 is identical to int.  These operations are *)
(* included mostly for compatibility with other versions of ML *)
val of_int: int -> int32
val to_int: int32 -> int

(* Conversions to integers are to either the same size or same sign *)
val of_uint32: uint32 -> int32
val to_uint32: int32 -> uint32

val of_int64: int64 -> int32
val to_int64: int32 -> int64

val of_nativeint: nativeint -> int32
val to_nativeint: int32 -> nativeint

val of_string: string -> int32
val to_string: int32 -> string

val float_of_bits: int32 -> float
val bits_of_float: float -> int32

val float32_of_bits: int32 -> float32
val bits_of_float32: float32 -> int32

