(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.Int64

let compare (x:int64) y = Pervasives.compare x y

let zero = (# "ldc.i8 0" : int64) 
let one = (# "ldc.i8 1" : int64) 
let minus_one = (# "ldc.i8 -1" : int64) 
let neg (x:int64) =  (# "neg" x : int64) 
let add (x:int64) (y:int64) = (# "add" x y : int64) 
let sub (x:int64) (y:int64) = (# "sub" x y : int64) 
let mul (x:int64) (y:int64) = (# "mul" x y : int64)
let div (x:int64) (y:int64) = (# "div" x y : int64)
let rem (x:int64) (y:int64) = (# "rem" x y : int64)
let succ (x:int64) = (# "ldc.i8 1 add" x : int64)
let pred (x:int64) = (# "ldc.i8 1 sub" x : int64)
let abs (x:int64) = if x < zero then neg x else x
let max_int = (# "ldc.i8 9223372036854775807" : int64)
let min_int = (# "ldc.i8 -9223372036854775808" : int64)
let logand (x:int64) (y:int64) = (# "and" x y : int64)
let logor (x:int64) (y:int64) = (# "or" x y : int64)
let logxor (x:int64) (y:int64) = (# "xor" x y : int64)
let lognot (x:int64) = (# "not" x : int64)
let shift_left (x:int64) (n:int) =  (# "shl" x n : int64)
let shift_right (x:int64) (n:int) =  (# "shr" x n : int64)
let shift_right_logical (x:int64) (n:int) =  (# "shr.un" x n : int64)
let of_int (n:int) =  (# "conv.i8" n : int64)
let to_int (x:int64) = (# "conv.i4" x : int)
let of_int32 (n:int32) =  (# "conv.i8" n : int64)
let to_int32 (x:int64) = (# "conv.i4" x : int32)
let of_uint64 (n:uint64) =  (# "conv.i8" n : int64)
let to_uint64 (x:int64) = (# "conv.u8" x : uint64)
let of_nativeint (n:nativeint) =  (# "conv.i8" n : int64)
let to_nativeint (x:int64) = (# "conv.i" x : nativeint)
let of_float (f:float) =  (# "conv.i8" f : int64)
let to_float (x:int64) =  (# "conv.r8" x : float)
let of_string (s:string) = 
 try
  if (String.length s >= 3 & s.[0] = '-' & s.[1] = '0' & s.[2] = 'x') then 
    System.Int64.Parse(String.sub s 3 (String.length s - 3), 
                       System.Globalization.NumberStyles.AllowHexSpecifier,
                       System.Globalization.CultureInfo.InvariantCulture)
  else if (String.length s >= 2 & s.[0] = '0' & s.[1] = 'x') then
    System.Int64.Parse(String.sub s 2 (String.length s - 2), 
                       System.Globalization.NumberStyles.AllowHexSpecifier,
                       System.Globalization.CultureInfo.InvariantCulture)
  else 
    System.Int64.Parse(s,System.Globalization.CultureInfo.InvariantCulture)
 with :? System.FormatException -> failwith "Int32.of_string"
let to_string (x:int64) = (box x).ToString()

(* Yes folks, the only verifiable way to get the bits from a float *)
(* is to blast it to a stream then read it back in... *)
let bits_of_float (x:float) = 
   let ms = new System.IO.MemoryStream() in 
   let bw = new System.IO.BinaryWriter(ms) in 
   bw.Write(x); 
   ms.Flush();
   let br = new System.IO.BinaryReader(ms) in 
   ignore(ms.Seek(Int64.of_int 0, System.IO.SeekOrigin.Begin)); 
   let res = br.ReadInt64() in 
   br.Close(); 
   res

let float_of_bits (x:int64) = 
   let ms = new System.IO.MemoryStream() in 
   let bw = new System.IO.BinaryWriter(ms) in 
   bw.Write(x); 
   ms.Flush();
   let br = new System.IO.BinaryReader(ms) in 
   ignore(ms.Seek(Int64.of_int 0, System.IO.SeekOrigin.Begin)); 
   let res = br.ReadDouble() in 
   br.Close(); 
   res


let of_float32 (f:float32) =  (# "conv.i8" f : int64)
let to_float32 (x:int64) =  (# "conv.r4" x : float32)



