(*==========================================================================
 * (c) Microsoft Corporation. All rights reserved 
 *
 *=========================================================================*)

module Microsoft.FSharp.MLLib.Lexing

type position = {
  pos_fname : string;
  pos_lnum : int;
  pos_bol : int;
  pos_cnum : int;
}
let dummy_pos = {pos_fname=""; pos_lnum= 0; pos_bol= 0; pos_cnum=0 }

type lexbuf = (position,byte) Primitives.Lexing.LexBuffer

let getStartMark (lb : lexbuf) = Primitives.Lexing.GetStartMark lb 
let getEndMark (lb : lexbuf) = Primitives.Lexing.GetEndMark lb 

let new_lexbuf lb = 
  Primitives.Lexing.SetMatchEvent lb
    (fun () -> 
      let curr = getEndMark lb in 
      Primitives.Lexing.SetStartMark lb curr;
      Primitives.Lexing.SetEndMark lb { curr with pos_cnum=curr.pos_cnum + Primitives.Lexing.Ascii.GetLexemeLength lb });
  lb

let from_function f = new_lexbuf (Primitives.Lexing.Ascii.FromFunction dummy_pos f)
let from_channel is  = from_function (fun s max -> Pervasives.input is s 0 max)
let from_stream_reader (sr: System.IO.StreamReader)  = from_function (fun s max -> sr.BaseStream.Read(s,0,max))
let from_bytearray s  = new_lexbuf (Primitives.Lexing.Ascii.FromBytes dummy_pos s)
let from_string s  = from_bytearray (Bytearray.string_to_ascii s)

let lexeme_char (lb:lexbuf) n =  Char.chr (Byte.to_int (Primitives.Lexing.Ascii.GetLexemeByte lb n))
let lexeme_start_p lb = getStartMark lb
let lexeme_end_p lb = getEndMark lb
let lexeme_start lb = (lexeme_start_p lb).pos_cnum
let lexeme_end lb = (lexeme_end_p lb).pos_cnum
let lexeme (lb:lexbuf) = Bytearray.ascii_to_string (Primitives.Lexing.Ascii.GetLexeme lb)
let flush_input (lb: lexbuf) = Primitives.Lexing.Ascii.DiscardInput lb

let lexbuf_curr_p lb = lexeme_end_p lb
let lexbuf_set_curr_p (lb:lexbuf) (p : position) = Primitives.Lexing.SetEndMark lb p

