(*==========================================================================
 * Basic operations on options.  
 *
 * (c) Microsoft Corporation 2004.  All code and documentation in this file 
 * has been written 'from scratch'. 
 *=========================================================================*)


module Microsoft.FSharp.MLLib.Option

let get = function None -> invalid_arg "Option.get" | Some x -> x
let length = function None -> 0 | Some _ -> 1
let nonempty = function None -> false | Some _ -> true
let fold_left f s inp = match inp with None -> s | Some x -> f s x
let fold_right f inp s =  match inp with None -> s | Some x -> f x s

let exists p inp = match inp with None -> false | Some x -> p x
let for_all p inp = match inp with None -> true | Some x -> p x
let filter p inp = match inp with None -> None | Some x -> if p x then inp else None
let partition p inp = match inp with None -> None,None | Some x -> if p x then inp,None else None,inp
let iter f inp = match inp with None -> () | Some x -> f x
let map f inp = match inp with None -> None | Some x -> Some (f x)

// These are like the above but with the object(s) being iterated placed first.
let foreach x f = iter f x
let transform x f = map f x

(*--------------------------------------------------------------------------
 * The following functions relate streams to the most common other F# 
 * collection types.
 *--------------------------------------------------------------------------*)

let to_array = function None -> [| |] | Some x -> [| x |]
let to_stream = function None -> Stream.empty() | Some x -> Stream.cons x (Stream.empty())
let to_list = function None -> [ ] | Some x -> [ x ]
