(*==========================================================================
 * Basic operations on options.  
 *
 * (c) Microsoft Corporation 2004.  All code and documentation in this file 
 * has been written 'from scratch'.  The interface to the module 
 * is similar to that found in versions of other ML implementations, 
 * but is not an exact match.  The type signatures in this interface
 * are an edited version of those generated automatically by running 
 * "bin\fsc.exe -i" on the implementation file.
 *=========================================================================*)

module Microsoft.FSharp.MLLib.Option


///Gets the value associated with the option.  If the option is None then
///raises Invalid_argument "Option.get"
val get: 'a option -> 'a

///"length inp" evaluates to "match inp with None -> 0 | Some _ -> 1"
val length: 'a option -> int

///"nonempty inp" evaluates to "match inp with None -> false | Some _ -> true"

///"fold_left f s inp " evaluates to "match inp with None -> s | Some x -> f s x"
val fold_left: ('b -> 'a -> 'b) -> 'b -> 'a option -> 'b

///"fold_right f inp s" evaluates to "match inp with None -> s | Some x -> f x s"
val fold_right: ('a -> 'b -> 'b) -> 'a option -> 'b -> 'b

///"exists p inp" evaluates to "match inp with None -> false | Some x -> p x"
val exists: ('a -> bool) -> 'a option -> bool

///"forall p inp" evaluates to "match inp with None -> true | Some x -> p x"
val for_all: ('a -> bool) -> 'a option -> bool

///"filter p inp" evaluates to "match inp with None -> None | Some x -> if p x then inp else None"
val filter: ('a -> bool) -> 'a option -> 'a option

///"partition p inp" evaluates to 
///"match inp with None -> None,None | Some x -> if p x then inp,None else None,inp"
val partition: ('a -> bool) -> 'a option -> ('a option * 'a option)

///"iter f inp" executes "match inp with None -> () | Some x -> f x"
val iter: ('a -> unit) -> 'a option -> unit

///"map f inp" evaluates to "match inp with None -> None | Some x -> Some (f x)"
val map: ('a -> 'b) -> 'a option -> 'b option

///Apply the given function to each element of the collection. Options have at most one item when viewed as a collection
val foreach: 'a option -> ('a -> unit) -> unit

///Build a new collection whose elements are the results of applying the given function
///to each of the elements of the collection. Options have at most one item when viewed as a collection.
val transform: 'a option -> ('a -> 'b) -> 'b option

(*--------------------------------------------------------------------------
 * The following functions relate streams to the most common other F# 
 * collection types.
 *--------------------------------------------------------------------------*)

///Convert the option to an array of length 0 or 1
val to_array: 'a option -> 'a array
///Convert the option to a stream of length 0 or 1
val to_stream: 'a option -> 'a Stream.t
///Convert the option to a list of length 0 or 1
val to_list: 'a option -> 'a list

