(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.Pervasives

exception Match_failure = Microsoft.FSharp.MatchFailureException
exception Assert_failure = Microsoft.FSharp.AssertionFailureException

exception End_of_file = System.IO.EndOfStreamException
exception Out_of_memory = System.OutOfMemoryException
exception Division_by_zero = System.DivideByZeroException
exception Stack_overflow = System.StackOverflowException 
exception Not_found = System.IndexOutOfRangeException
exception Failure of string
exception Invalid_argument of string

type ('a,'b,'c,'d) format4 = ('a,'b,'c,'d) Microsoft.FSharp.Primitives.PrintfPrimitives.Format4
type ('a,'b,'c) format = ('a,'b,'c,'c) format4
type 'a ref = Microsoft.FSharp.Ref<'a>
type 'a option = Microsoft.FSharp.Option<'a>
type 'a list = Microsoft.FSharp.List<'a>
type exn = System.Exception

let unbox (x:obj) = Primitives.Operators.op_Unbox x
let inline raise e = Primitives.Operators.op_Raise e
let box (x:'a) = Primitives.Operators.op_Box x
let (=) (x : 'a) (y : 'a) = Primitives.Operators.op_StructuralEquality x y
let int_neg (x:int) = Primitives.Operators.op_IntegerNegation x
let not (b:bool) = Primitives.Operators.op_BooleanNegation b
let compare (x : 'a) (y : 'a) = Primitives.Operators.op_StructuralComparison x y
let (<>) (x:'a) (y:'a) = Primitives.Operators.op_StructuralInequality x y 
let (<) (x:'a) (y:'a) = Primitives.Operators.op_StructuralLessThan x y
let (>) (x:'a) (y:'a) = Primitives.Operators.op_StructuralGreaterThan x y
let (<=) (x:'a) (y:'a) = Primitives.Operators.op_StructuralLessThanOrEqual x y
let (>=) (x:'a) (y:'a) = Primitives.Operators.op_StructuralGreaterThanOrEqual x y
let min x y = Primitives.Operators.op_StructuralMinimum x y
let max x y = Primitives.Operators.op_StructuralMaximum x y
let (==) (x: 'a)  (y: 'a) = Primitives.Operators.op_GenericEq x y
let (!=) (x:'a) (y:'a) = Primitives.Operators.op_GenericNotEq x y

(*

let (~-) (x:int) = Primitives.Operators.op_IntegerNegation x
let (~+) x = Primitives.Operators.op_IntegerNegation x
let (+) (x:int) (y:int) = Primitives.Operators.op_IntegerAddition x y
let (-) (x:int) (y:int) = Primitives.Operators.op_IntegerSubtraction x y
let ( * ) (x:int) (y:int) = Primitives.Operators.op_IntegerMultiplication x y
let (/) (x:int) (y:int) = Primitives.Operators.op_IntegerDivision x y
*)

let inline (~+) x = Microsoft.FSharp.Primitives.Operators.op_OverloadedUnaryPlus x
let inline (~-) x = Microsoft.FSharp.Primitives.Operators.op_OverloadedUnaryNegation x 
let inline (+) x y = Microsoft.FSharp.Primitives.Operators.op_OverloadedAddition x y
let inline (-) x y = Microsoft.FSharp.Primitives.Operators.op_OverloadedSubtraction x y
let inline ( * ) x y = Microsoft.FSharp.Primitives.Operators.op_OverloadedMultiplication x y
let inline (/) x y = Microsoft.FSharp.Primitives.Operators.op_OverloadedDivision x y
let inline (mod) x y = Microsoft.FSharp.Primitives.Operators.op_OverloadedModulus x y


let inline (land) (x) (y) = Primitives.Operators.op_OverloadedBitwiseAnd x y
let inline (lor) (x) (y) = Primitives.Operators.op_OverloadedBitwiseOr x y
let inline (lxor) (x) (y) = Primitives.Operators.op_OverloadedExclusiveOr x y
let inline lnot (x) = Primitives.Operators.op_OverloadedLogicalNot x
let inline (lsl) (x) (y) = Primitives.Operators.op_OverloadedLeftShift x y
let inline (lsr) (x) (y) = Primitives.Operators.op_OverloadedLogicalRightShift x y
let inline (asr) (x) (y) = Primitives.Operators.op_OverloadedRightShift x y

let (~-.) (x:float) =  Primitives.Operators.op_FloatNegation x
let (~+.) (x:float) =  x
let (+.) (x:float) (y:float) =  Primitives.Operators.op_FloatAddition x y
let (-.) (x:float) (y:float) =  Primitives.Operators.op_FloatSubtraction x y
let ( *.) (x:float) (y:float) =  Primitives.Operators.op_FloatMultiplication x y
let ( /.) (x:float) (y:float) =  Primitives.Operators.op_FloatDivision x y
let ( ** ) (x:float) (y:float) =  Primitives.Operators.op_FloatExponentiation x y

let inline (.()) arr n = Primitives.Operators.op_ArrayGet arr n
let inline (.()<-) arr n x = Primitives.Operators.op_ArraySet arr n x
let (.[]) s n = Primitives.Operators.op_StringGet s n

let inline (.(,)) (arr: 'a[,]) ((n1:int),(n2:int)) = (# "ldelem.multi 2 !0" type ('a) arr n1 n2 : 'a)  
let inline (.(,,)) (arr: 'a[,,]) ((n1:int),(n2:int),(n3:int)) = (# "ldelem.multi 3 !0" type ('a) arr n1 n2 n3 : 'a)  
let inline (.(,)<-) (arr: 'a[,]) ((n1:int),(n2:int)) (x:'a) = (# "stelem.multi 2 !0" type ('a) arr n1 n2 x)  
let inline (.(,,)<-) (arr: 'a[,,]) ((n1:int),(n2:int),(n3:int)) (x:'a) = (# "stelem.multi 3 !0" type ('a) arr n1 n2 n3 x)  


let isinst (x:obj) = (unbox (# "isinst !0" type ('a) x : obj) : 'a)
let nonnull (x:'a) = (# "ldnull cgt.un" (box x) : bool)

let inline invalid_arg s = raise (Invalid_argument s)
let inline failwith s = raise (Failure s)
let succ (x:int) = x + 1
let pred (x:int) = x - 1
let abs (x:int) = if x < 0 then int_neg x else x
let max_int = 0x7FFFFFFF (* 2147483647 *)
let min_int = 0x80000000 (* -2147483648 *)

let sqrt (x:float) = System.Math.Sqrt(x)
let exp (x:float) = System.Math.Exp(x)
let log (x:float) = System.Math.Log(x)
let log10 (x:float) = System.Math.Log10(x)
let cos (x:float) = System.Math.Cos(x)
let sin (x:float) = System.Math.Sin(x)
let tan (x:float) = System.Math.Tan(x)
let acos (x:float) = System.Math.Acos(x)
let asin (x:float) = System.Math.Asin(x)
let atan (x:float) = System.Math.Atan(x)
let atan2 (x:float) (y:float) = System.Math.Atan2(x,y)
let cosh (x:float) = System.Math.Cosh(x)
let sinh (x:float) = System.Math.Sinh(x)
let tanh (x:float) = System.Math.Tanh(x)
let ceil (x:float) = System.Math.Ceiling(x)
let floor (x:float) = System.Math.Floor(x)
let abs_float (x:float) = System.Math.Abs(x)
let mod_float (x:float) (y:float) = System.Math.IEEERemainder(x,y)

let float (x:int) =  (# "conv.r8" x : float)
let float_of_int (x:int) =  (# "conv.r8" x : float)

let ldexp x n = x *. (2.0 ** float_of_int n)
let modf x = let integral = floor x in (integral, x -. integral)
let truncate (x:float) =  System.Convert.ToInt32(if x < 0. then ceil x else floor x)
let int_of_float x =  truncate x

let float32_of_int (n:int) =  (# "conv.r4" n : float32)
let float32 (n:int) =  (# "conv.r4" n : float32)
let int_of_float32 (x:float32) = (# "conv.i4" x : int)

let string_of_float32 (x:float32) = (box x).ToString()
let float32_of_string (s:string) = 
  try System.Single.Parse(s,System.Globalization.CultureInfo.InvariantCulture)
  with :? System.FormatException -> failwith "Float32.of_string"


let infinity = System.Double.PositiveInfinity
let neg_infinity = System.Double.NegativeInfinity
let max_float = System.Double.MaxValue
let min_float = System.Double.MinValue
let nan = System.Double.NaN 

type fpclass = FP_normal (* | FP_subnormal *) | FP_zero| FP_infinite | FP_nan      

(* TODO: subnormal never returned *)
let classify_float (x:float) = 
  if System.Double.IsNaN(x) then FP_nan
  else if System.Double.IsNegativeInfinity(x) then FP_infinite
  else if System.Double.IsPositiveInfinity(x) then FP_infinite
  else if x = 0.0 then FP_zero
  else FP_normal
       
let (^) (x:string) (y:string) = System.String.Concat(x,y)

let int_of_char (c:char) = (# "" c : int)

let char_of_int (i:int) = 
    if i > 65535 then
       invalid_arg "char_of_int" 
    else if i < 0 then 
       invalid_arg "char_of_int: negative" 
    else  (# "" i : char)

let max_char = char_of_int 65535
let min_char = char_of_int 0


let ignore x = ()
let string_of_bool b = if b then "true" else "false"
let bool_of_string s = if s = "true" then true else if s = "false" then false else invalid_arg "bool_of_string"

let fst (x,y) = x
let snd (x,y) = y

let ref x = { contents=x }
let (!) x = x.contents
let (:=) x y = x.contents <- y

let string_of_int (x:int) = x.ToString()

let string_get (s:string) (n:int) = s.Chars(n)
let string_length (s:string) =  s.Length
let string_sub (s:string) (start:int) (len:int) =  s.Substring(start,len)

let int_of_string (s:string) = 
  try
    let l = string_length s in 
    if 
      (if (l >= 3) then 
        if string_get s 0 = '-'  then 
	  if string_get s 1 = '0' then string_get s 2 = 'x' else false
        else false
      else false) then
      System.Int32.Parse(string_sub s 3 (l - 3), 
                         System.Globalization.NumberStyles.AllowHexSpecifier,
                         System.Globalization.CultureInfo.InvariantCulture)
    else if 
      (if (l >= 2) then 
        if string_get s 0 = '0' then string_get s 1 = 'x' else false
      else false) then
      System.Int32.Parse(string_sub s 2 (l - 2), 
                         System.Globalization.NumberStyles.AllowHexSpecifier,
                         System.Globalization.CultureInfo.InvariantCulture)
    else 
      System.Int32.Parse(s,System.Globalization.CultureInfo.InvariantCulture)
  with :? System.FormatException -> failwith "int_of_string"
let string_of_float (x:float) = x.ToString()
let float_of_string (s:string) = 
  try System.Double.Parse(s,System.Globalization.CultureInfo.InvariantCulture)
  with :? System.FormatException -> failwith "float_of_string"
              

let (@) l1 l2 = Primitives.Operators.op_ListAppend l1 l2

(*--------------------------------------------------------------------------
 * I/O.  
 *
 * Notes on the implementation: We discriminate between three kinds of 
 * readers/writers since various operations are possible on each kind.
 * StreamReaders/StreamWriters inherit from text readers/writers and
 * thus support more functionality.  We could just support two 
 * constructors (Binary and Text) and use dynamic type tests on the underlying .NET
 * objects to detect the cases where we have StreamWriters.
 *--------------------------------------------------------------------------*)

open System.IO

type writer = 
  | StreamW of StreamWriter
  | TextW of TextWriter
  | BinaryW of BinaryWriter
type out_channel = OutChannel of writer ref
type open_flag =
  | Open_rdonly | Open_wronly | Open_append
  | Open_creat | Open_trunc | Open_excl
  | Open_binary | Open_text | Open_nonblock
  | Open_encoding of System.Text.Encoding

type reader = 
  | StreamR of StreamReader
  | TextR of TextReader
  | BinaryR of BinaryReader
type in_channel = InChannel of reader ref

let (!!) (OutChannel os) = !os
let (<--) (OutChannel os) os' = os := os'

let get_stdout () = System.Console.Out
let get_stderr () = System.Console.Error
let stdout =  OutChannel (ref (TextW (get_stdout ())))
let stderr =  OutChannel (ref (TextW (get_stderr ())))

(* This default encoding gives identical behaviour for OCaml and F# *)
let stream_to_binary_writer s    = BinaryW (new BinaryWriter(s))
let stream_to_stream_writer (encoding :> System.Text.Encoding) (s :> Stream) =   StreamW (new StreamWriter(s,encoding))
let stream_to_out_channel encoding (s :> Stream) =   OutChannel (ref (stream_to_stream_writer encoding s))


let rec mem x l = match l with [] -> false | h::t -> if x = h then true else mem x t
let rec chooseList f l = match l with [] -> None | h::t -> match f h with None -> chooseList f t | Some r -> Some r

let open_out_gen flags (perm:int) (s:string) = 
  (* TODO: permissions are ignored *)
  let app = mem Open_append flags in 
  let access = 
    match mem Open_rdonly flags, mem Open_wronly flags with
    | true, true -> invalid_arg "open_out_gen: access"
    | true, false -> invalid_arg "open_out_gen: invalid access for writing" (* FileAccess.Read *)
    | false, true ->  FileAccess.Write
    | false, false -> (if app then FileAccess.Write else FileAccess.ReadWrite)  in 
  let mode =
    match (mem Open_excl flags,app, mem Open_creat flags, mem Open_trunc flags) with
    | (true,false,false,false) -> FileMode.CreateNew
    | false,false,true,false -> FileMode.Create
    | false,false,false,false -> FileMode.OpenOrCreate
    | false,false,false,true -> FileMode.Truncate
    | false,true,false,false -> FileMode.Append
    | _ -> invalid_arg "open_out_gen: mode" in 
  let share = FileShare.Read in 
  let buf_size = 0x1000 in 
  let allow_async = mem Open_nonblock flags in 
  let stream = (new FileStream(s,mode,access,share,buf_size,allow_async)) in
  match mem Open_binary flags, mem Open_text flags with 
  | true,true -> invalid_arg "open_out_gen: text/binary"
  | true,false -> OutChannel (ref (stream_to_binary_writer stream ))
  | false,_ ->
      let encoding = chooseList (function Open_encoding e -> Some(e) | _ -> None) flags in 
      let encoding = match encoding with None -> System.Text.Encoding.Default | Some e -> e in
      stream_to_out_channel encoding stream
        
let open_out_encoded e (s:string) = open_out_gen [Open_text; Open_wronly; Open_creat; Open_encoding (e :> System.Text.Encoding)] 777 s
let open_out_utf8 (s:string) = open_out_encoded System.Text.Encoding.UTF8 s
let open_out (s:string) = open_out_gen [Open_text; Open_wronly; Open_creat] 777 s
let open_out_bin (s:string) = open_out_gen [Open_binary; Open_wronly; Open_creat] 777 s

let flush os = 
  match !!os with 
  | TextW tw   -> tw.Flush() (* the default method does not flush, is it overriden for the console? *)
  | BinaryW bw -> bw.Flush()
  | StreamW sw -> sw.Flush()

let close_out os = 
  match !!os with 
  | TextW tw -> tw.Close()
  | BinaryW bw -> bw.Close()
  | StreamW sw -> sw.Close()

let output_string os (s:string) = 
  match !!os with 
  | TextW tw -> tw.Write(s)
  | BinaryW bw -> bw.Write(s)
  | StreamW sw -> sw.Write(s)

let output_char os (c:char) = 
  match !!os with 
  | TextW tw -> tw.Write(c)
  | BinaryW bw -> bw.Write(c)
  | StreamW sw -> sw.Write(c)

let out_channel_to_stream os =
  match !!os with 
  | TextW tw -> failwith "out_channel_to_stream: cannot access a stream for this channel"
  | BinaryW bw -> bw.BaseStream 
  | StreamW sw -> sw.BaseStream
  
let out_channel_to_stream_writer os =
  match !!os with 
  | StreamW sw -> sw
  | _ -> failwith "out_channel_to_stream_writer: cannot access a stream writer for this channel"
  
let out_channel_to_text_writer os =
  match !!os with 
  | StreamW sw -> (sw :> TextWriter)
  | TextW tw -> tw
  | _ -> failwith "out_channel_to_text_writer: cannot access a text writer for this channel"
  
let stream_writer_to_out_channel w =
  OutChannel (ref (StreamW(w :> StreamWriter)))

let out_channel_to_binary_writer os =
  match !!os with 
  | BinaryW bw -> bw
  | _ -> failwith "out_channel_to_binary_writer: cannot access a binary writer for this channel"
  
let binary_writer_to_out_channel (w :> BinaryWriter) =
  OutChannel (ref (BinaryW (w :> BinaryWriter)))
  
let text_writer_to_out_channel (w:> TextWriter) =
  let absw = 
    match (w :> TextWriter) with 
    | :? StreamWriter as sw -> StreamW sw
    | tw -> TextW tw in
  OutChannel (ref absw) 
    
let stream_reader_to_in_channel w =
  InChannel (ref (StreamR(w :> StreamReader)))

let binary_reader_to_in_channel r =
  InChannel (ref (BinaryR (r :> BinaryReader)))
  
let text_reader_to_in_channel (r:> TextReader) =
  let absr = 
    match (r :> TextReader) with 
    | :? StreamReader as sr -> StreamR sr
    | tr -> TextR tr in
  InChannel (ref absr)
  
let int_to_int64 (n:int) = (# "conv.i8" n : int64)
let int64_to_int (n:int64) = (# "conv.i4" n : int)

let seek_out os (n:int) = 
  match !!os with 
  | StreamW sw -> 
      sw.Flush();   
      ignore((out_channel_to_stream os).Seek(int_to_int64 n,SeekOrigin.Begin))
  | TextW _ ->
      ignore((out_channel_to_stream os).Seek(int_to_int64 n,SeekOrigin.Begin))
  | BinaryW bw -> 
      bw.Flush();
      ignore (bw.Seek(n,SeekOrigin.Begin))

let pos_out os = flush os; int64_to_int (out_channel_to_stream os).Position
let out_channel_length os = flush os; int64_to_int (out_channel_to_stream os).Length

let output (os:out_channel) (buf: byte[]) (x:int) (len: int) = 
  match !!os with 
  | BinaryW bw -> bw.Write(buf,x,len)
  | TextW tw -> 
      for i = x to x + len - 1 do 
        tw.Write(# "ldelem.i1" buf i : char)
      done
  | StreamW sw -> 
     sw.Write(System.Text.Encoding.ASCII.GetString(buf,x,len))

let output_bytearray os (b: byte[]) = output os b 0  (# "ldlen" b : int)  


let byte_to_int (x:byte) = (# "" x : int)
let int_to_byte (x:int) = (# "conv.u1" x : byte)
let int_to_char (x:int) = (# "conv.u2" x : char)

let output_byte os (x:int) = 
  match !!os with 
  | TextW tw ->  tw.Write(int_to_char (x mod 256))
  | BinaryW bw -> bw.Write(int_to_byte (x mod 256))
  | StreamW sw -> sw.Write(int_to_char (x mod 256))

let output_binary_int os (x:int) = 
  match !!os with 
  | BinaryW bw -> bw.Write(x)
  | _ -> failwith "output_binary_int: not a binary stream"

let set_binary_mode_out os b = 
  match !!os with 
  | StreamW _ when not b -> ()
  | BinaryW _ when b -> ()
  | BinaryW bw -> os <--  stream_to_stream_writer System.Text.Encoding.Default (out_channel_to_stream os)
  | StreamW bw -> os <-- stream_to_binary_writer (out_channel_to_stream os)
  | TextW _ when b -> failwith "cannot set this stream to binary mode"
  | TextW _ -> ()

let print_string (s:string) = System.Console.Write(s)
let print_newline () = System.Console.WriteLine()
let print_int (x:int) = System.Console.Write(x)
let print_float (x:float) = System.Console.Write(x)
let print_endline (s:string) = System.Console.WriteLine(s)
let print_char (c:char) = System.Console.Write(c)

let output_value os (x: 'a) = 
  let formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter() in
  flush os;
  formatter.Serialize(out_channel_to_stream os,box [x]);
  flush os

let prerr_int (x:int) = (get_stderr()).Write(x)
let prerr_float (x:float) = (get_stderr()).Write(x)
let prerr_string (x:string) = (get_stderr()).Write(x)
let prerr_endline (x:string) = (get_stderr()).WriteLine(x)
let prerr_newline () = (get_stderr()).WriteLine()
let prerr_char (c:char) = (get_stderr()).Write(c)

let (!!!) (InChannel isref) = !isref
let (<---) (InChannel isref) is' = isref := is'

let get_stdin () = System.Console.In
let stdin = InChannel (ref (TextR (get_stdin())))

let stream_to_binary_reader (s:>Stream) = BinaryR (new BinaryReader(s))

let stream_to_stream_reader e (s:>Stream) = StreamR (new StreamReader(s, e,false))
let stream_to_in_channel e (s:>Stream) =   InChannel (ref (stream_to_stream_reader e s))

(* TODO: permissions are ignored *)
let open_in_gen flags (perm:int) (s:string) = 
  let access = 
    match mem Open_rdonly flags, mem Open_wronly flags with
    | true, true -> invalid_arg "open_in_gen: access"
    | true, false -> FileAccess.Read 
    | false, true -> invalid_arg "open_in_gen: invalid access for reading"
    | false, false -> FileAccess.ReadWrite in 
  let mode = 
    match mem Open_excl flags,mem Open_append flags, mem Open_creat flags, mem Open_trunc flags with
    | false,false,false,false -> FileMode.Open
    | _ -> invalid_arg "open_in_gen: invalid mode for reading" in
  let share = FileShare.Read in 
  let buf_size = 0x1000 in 
  let allow_async = mem Open_nonblock flags in 
  let stream = new FileStream(s,mode,access,share,buf_size,allow_async) :> Stream in 
  match mem Open_binary flags, mem Open_text flags with 
  | true,true -> invalid_arg "open_in_gen: text/binary"
  | true,false -> InChannel (ref (stream_to_binary_reader stream ))
  | false,_ ->
      let encoding = chooseList (function Open_encoding e -> Some(e) | _ -> None) flags in 
      let encoding = match encoding with None -> System.Text.Encoding.Default | Some e -> e in
      stream_to_in_channel encoding stream
  
let open_in_encoded e (s:string) = open_in_gen [Open_text; Open_rdonly; Open_encoding (e :> System.Text.Encoding)] 777 s
let open_in_utf8 (s:string) = open_in_encoded System.Text.Encoding.UTF8 s
let open_in (s:string) = open_in_gen [Open_text; Open_rdonly] 777 s
let open_in_bin (s:string) = open_in_gen [Open_binary; Open_rdonly] 777 s

let close_in is = 
  match !!!is with 
  | TextR tw -> tw.Close()
  | BinaryR bw -> bw.Close()
  | StreamR sw -> sw.Close()

let input_line is = 
  match !!!is with 
  | BinaryR bw -> failwith "input_line: binary mode"
  | TextR tw -> tw.ReadLine() ?? raise End_of_file
  | StreamR sw -> sw.ReadLine() ?? raise End_of_file

let input_byte is = 
  match !!!is with 
  | BinaryR bw ->  byte_to_int (bw.ReadByte())
  | TextR tr -> let b = tr.Read() in if b = -1 then raise End_of_file else byte_to_int (int_to_byte b)
  | StreamR sr -> let b = sr.Read() in if b = -1 then raise End_of_file else byte_to_int (int_to_byte b)

let input_char is = char_of_int (input_byte is)

let in_channel_to_stream_reader is =
  match !!!is with 
  | StreamR sr -> sr
  | _ -> failwith "in_channel_to_stream_reader: cannot access a stream reader for this channel"
  
let in_channel_to_binary_reader is =
  match !!!is with 
  | BinaryR sr -> sr
  | _ -> failwith "in_channel_to_binary_reader: cannot access a binary reader for this channel"
  
let in_channel_to_text_reader is =
  match !!!is with 
  | TextR sr -> sr
  | _ -> failwith "in_channel_to_text_reader: cannot access a text reader for this channel"
  
let in_channel_to_stream is =
  match !!!is with 
  | TextR tw -> failwith "cannot seek, set position or calculate length of this stream"
  | BinaryR bw -> bw.BaseStream
  | StreamR sw -> sw.BaseStream

let seek_in is (n:int) = 
  begin match !!!is with 
  | StreamR sw -> sw.DiscardBufferedData() 
  | TextR _ | BinaryR _ -> ()
  end;
  ignore ((in_channel_to_stream is).Seek(int_to_int64 n,SeekOrigin.Begin))

let pos_in is  = int64_to_int (in_channel_to_stream is).Position
let in_channel_length is  = int64_to_int (in_channel_to_stream is).Length

let input (is: in_channel) (buf: byte[]) (x:int) (len: int) = 
  match !!!is with 
  | StreamR _  -> (in_channel_to_stream is).Read(buf,x,len)
  | TextR tr   -> let charbuf : char[] = Microsoft.FSharp.Compatibility.CompatArray.zero_create len in
                  let nRead = tr.Read(charbuf,x,len) in
		  (* Borrow some functions from byte.fs *)
		  let byte_of_char (n:char) = (# "conv.u1" n : byte) in
		  let int_of_byte (x:byte)  = (# "" x : int) in
		  for i=0 to nRead-1 do
		    let c  = Microsoft.FSharp.Compatibility.CompatArray.get charbuf i in
		    let bc = byte_of_char c in		    
		    if int_of_char c = int_of_byte bc then
                      Microsoft.FSharp.Compatibility.CompatArray.set buf i bc
		    else
		      failwith "input: in_channel (in this case a TextReader) had char outside of byte range"
		  done;
		  nRead
  | BinaryR br -> br.Read(buf,x,len)

let really_input is (buf: byte[]) (x:int) (len: int) = 
  let mutable n = 0 in 
  let mutable i = 1 in 
  while (if i > 0 then n < len else false) do 
    i <- input is buf (x+n) (len-n);
    n <- n + i
  done
    
let unsafe_really_input is buf x len = really_input is buf x len

let input_binary_int is = 
  match !!!is with 
  | BinaryR bw -> bw.ReadInt32()
  | _ -> failwith "input_binary_int: not a binary stream"

let set_binary_mode_in is b = 
  match !!!is with 
  | StreamR _ when not b -> ()
  | BinaryR _ when b -> ()
  | BinaryR bw -> is <---  stream_to_stream_reader System.Text.Encoding.Default (in_channel_to_stream is)
  | StreamR bw -> is <--- stream_to_binary_reader (in_channel_to_stream is)
  | TextR _ when b -> failwith "set_binary_mode_in: cannot set this stream to binary mode"
  | TextR _ -> ()

let read_line () = System.Console.Out.Flush(); System.Console.ReadLine()
let read_int () = int_of_string (read_line())
let read_float () = float_of_string (read_line ())

let input_value is = 
  let formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter() in
  let res = formatter.Deserialize(in_channel_to_stream is) in 
  match ((unbox res) : 'a list) with 
  | [x] -> x
  | _ -> failwith "input_value: expected one item"

let incr x = x.contents <- x.contents + 1
let decr x = x.contents <- x.contents - 1

let inline exit (n:int) : $a = System.Environment.Exit(n); failwith "exit failure"

exception Exit

(* --------------------------------------------------------------------
 * Structural Hash Function
 * -------------------------------------------------------------------- *)
  
let hash (x: 'a) = Primitives.Operators.op_StructuralHash x

let hashq (x: 'a) = 
#if GENERICS
// System.Runtime.CompilerServices.RuntimeHelpers.GetHashCode is only
// available post- v1.0.  
   System.Runtime.CompilerServices.RuntimeHelpers.GetHashCode(box x)
#else   
  (# "call instance int32 [mscorlib]System.Object::GetHashCode()" (box x) : int32)
#endif

//-------------------------------------------------------------------------
// Use the general-purpose reflection support
// to build a generic pretty-printer.
//-------------------------------------------------------------------------

open Experimental.Reflection

let rec list_iter f l = 
  match l with [] -> () | h :: t -> f h; list_iter f t
  
let array_nonempty arr = 
  Microsoft.FSharp.Primitives.ArrayPrimitives.length arr > 0

#if GENERICS
let IsListType (ty:System.Type) =
  array_nonempty (ty.GetGenericArguments()) 
      &&  ty.GetGenericTypeDefinition() =  
       (Idioms.typeof() : Idioms.typ<int list>).result.GetGenericTypeDefinition()
#else
let IsListType     (ty:System.Type) = ty.FullName = "Microsoft.FSharp.List"	 
#endif

// We keep a context to help print recursive terms.
type ctxt = 
 { printed : System.Collections.Hashtable;
   mutable count: int }

#if CLI_FRAMEWORK_AT_MOST_RTM
let new_ctxt () = { printed= new System.Collections.Hashtable();   // XXX - code review - check default comparer is ok.
		                                                   // XXX - now concerned about this, since
								   // XXX - Pervasives.(=) (box 1) (box true) errors.
                    count = 0 }  
#else
let qkeycomparer =
  // This is a hash table comparer based on object identity.  The
  { new System.Collections.IEqualityComparer 
    with  Equals (x,y) = (x==y) 
    and GetHashCode(x)=hashq x } 
let new_ctxt () = { printed= new System.Collections.Hashtable(qkeycomparer);
                    count = 0 }
#endif

let gvi os x = 
  try GetValueInfo x
  with e -> 
    output_string os "\n\n<<< Error in GetValueInfo(): \n";
    output_string os (e.ToString());
    output_string os "\n\n>>>: \n";
    AbstractValue(box(x))

let rec goutput_obj (ctxt: ctxt) os (x:obj) = 
  match x with 
  | null -> 
      goutput_repr ctxt os  (GetValueInfo(x))
  | _ -> 
    if (ctxt.printed.Contains(x)) then begin 
      output_string os "..";
      //output_string os "#";
      //output_string os (string_of_int (unbox (ctxt.printed.Item(x)) : int))
    end else begin
      ctxt.count <- ctxt.count + 1;
      ctxt.printed.Add(x,ctxt.count);
      goutput_repr ctxt os  (GetValueInfo(x))
    end
and goutput_repr ctxt os repr = 
  match repr with 
  | TupleValue(vals) ->
    output_string os "(";
    begin match vals with 
    | [] -> ()
    | h::t -> 
      goutput_obj ctxt os h; 
      list_iter (fun v -> 
        output_string os ",";
        goutput_obj ctxt os v) t; 
    end;
    output_string os ")"; 
  | RecordValue(recd) ->
    let output_recd_item os (n,v) = 
      output_string os n;
      output_string os "=";
      goutput_obj ctxt os v in 
    output_string os "{";
    begin match recd with 
    | [] -> ()
    | h::t -> 
      output_recd_item os h; 
      list_iter (fun v -> output_string os ";"; output_recd_item os v) t; 
    end;
    output_string os "}"; 
  | ConstructorValue (ty,_,recd) when IsListType ty -> 
      output_string os "[";
      begin match recd with 
      | [(_,h);(_,t)] -> 
        goutput_obj ctxt os h;
        let rec output_tail t = 
          match GetValueInfo(t) with 
          | ConstructorValue(ty,nm,[(_,h2);(_,t2)]) -> 
           output_string os ";";
           goutput_obj ctxt os h2;
           output_tail t2
          | _ -> () in 
        output_tail t;
      | _ -> ()
      end;
      output_string os "]";
      
  | ConstructorValue(ty,nm,recd) ->
      goutput_recd_as_tuple ctxt os nm recd; 
  | ExceptionValue(ty,recd) ->
    goutput_recd_as_tuple ctxt os ty.Name recd; 
  | FunctionClosureValue(ty,obj) ->
    output_string os "<closure ";
    output_string os ty.Name;
    output_string os ">"
  | AbstractValue(obj) ->
    begin match obj with 
    | null -> output_string os "<null>"
    | _ -> 
       let ty = obj.GetType() in 
       output_string os "<"; 
       output_string os ty.Name; 
       output_string os ">"
    end
  | ExternalValue(obj) ->
    begin match obj with 
    | null -> output_string os "<null>"
    | :? string as s -> 
      output_string os "\""; 
      output_string os s;
      output_string os "\""; 
    | _ ->
	let str = try obj.ToString() with _ -> "?" in
	output_string os str
    end
  | UnitValue -> output_string os "()"

and goutput_recd_as_tuple ctxt os nm recd = 
  match recd with 
  | [] ->
    output_string os nm 
  | h::t -> 
    output_string os nm;
    output_string os "(";
    goutput_obj ctxt os (snd h); 
    list_iter (fun v -> output_string os ","; goutput_obj ctxt os (snd v)) t; 
    output_string os ")"

let output_obj os x = 
  goutput_obj (new_ctxt ()) os x
  
let output_any os x = 
  let ctxt = new_ctxt() in 
  let xobj = box x in 
  (match xobj with null -> () | _ -> ctxt.printed.Add(xobj,ctxt.count));
  goutput_repr ctxt os (GetValueInfo(x))

let any_to_string x = 
  let sw = new System.IO.StringWriter() in 
  let oc = (text_writer_to_out_channel (sw :> System.IO.TextWriter)) in
  output_any oc x;
  let sb = sw.GetStringBuilder() in 
  sb.ToString()
  
  
//#else

//let any_to_string x = (box x).ToString()
//let output_any os x = output_string os (any_to_string x)

//#endif
let print_any x = output_any stdout x
let prerr_any x = output_any stderr x


let inline rethrow() = unbox(# "rethrow ldnull" : System.Object)



type bigint = Microsoft.FSharp.Math.BigInt
type bignum = Microsoft.FSharp.Math.BigNum
type big_int = bigint
type num = bignum

type 'a byref = Microsoft.FSharp.byref<'a>

let internal_printers =
  let defaultPrinter (x:obj) = (print_any x;true) in
  ref [defaultPrinter]

let internal_printer (x:'a) =
  let x = box x in
  let rec loop = function
      [] -> ()
    | (printer::printers) -> if printer x then () else loop printers
  in
  loop !internal_printers

  

let (|>) x f = f x
let (<|) f x = f x
let (>>) f g x = g(f x)
let (<<) f g x = f(g x)
