(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.Printf

open Microsoft.FSharp.Primitives.PrintfPrimitives


// These abbreviations constrain the very generic "format4" type to 
// indicate how intermediary printers for %a and %t actions must work.
type ('a,'d) kbuffer_format  = ('a, Buffer.t,    unit,   'd) format4
type ('a,'d) kchannel_format = ('a, out_channel, unit,   'd) format4
type ('a,'d) kstring_format  = ('a, unit,        string, 'd) format4

// These indicate the final type returned
type 'a buffer_format  = ('a,unit)   kbuffer_format        
type 'a channel_format = ('a,unit)   kchannel_format       
type 'a string_format  = ('a,string) kstring_format        
type 'a bstring_format = ('a,string) kbuffer_format 

let ksprintf (f : string -> 'd) (fmt : ('a, unit, string,'d) format4)  = 
  let buf = new System.Text.StringBuilder() in 
  let outc (c:char) = ignore (buf.Append(c)) in 
  let outa (s:string) = ignore (buf.Append(s)) in 
  gprintf outa outc () (fun () -> f (buf.ToString())) fmt

let kprintf f fmt = ksprintf f fmt

let kbprintf f (buf: Buffer.t) (fmt : ('a, Buffer.t, unit,'d) format4)  = 
  let outc (c:char) = ignore (buf.Append(c)) in 
  let outa () = () in 
  gprintf outa outc buf f fmt

let kfprintf f (os: out_channel) fmt = 
  let outc (c:char) = output_char os c in 
  let outa () = () in 
  gprintf outa outc os f fmt

let kbsprintf f fmt  = 
  let buf = new System.Text.StringBuilder() in 
  kbprintf (fun _ -> f (buf.ToString())) buf fmt 

let sprintf fmt  = kprintf (fun x -> x) fmt
let failwithf fmt  = kprintf failwith fmt
let bfailwithf fmt  = kbsprintf failwith fmt
let bprintf buf fmt  = kbprintf (fun _ -> ()) buf fmt 
let bsprintf fmt  = kbsprintf (fun x -> x) fmt 
let fprintf os fmt  = kfprintf (fun _ -> ()) os fmt 
let printf fmt = fprintf stdout fmt
let eprintf fmt = fprintf stderr fmt

