(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.String

let compare (x:string) y = Pervasives.compare x y

let length (s:string) =  s.Length
let fast_get (s:string) (n:int) = s.get_Chars(n)
let get (s:string) (n:int) = 
  try s.get_Chars(n)
  with :? System.ArgumentException -> invalid_arg "String.get" 
let of_char (c:char) = System.Char.ToString(c)
let make (n: int) (c: char) : string = new System.String(c, n)
let concat sep (strings : string list) =  
  System.String.Join(sep, Microsoft.FSharp.Compatibility.CompatArray.of_list strings)

let sub (s:string) (start:int) (len:int) =  
  try s.Substring(start,len)
  with :? System.ArgumentException -> invalid_arg "String.sub" 

let iter (f : (char -> unit)) (s:string) =
  for i = 0 to length s - 1 do f (fast_get s i) done
  
let index_from (s:string) (start:int) (c:char) =  
  try let r = s.IndexOf(c,start) in if r = -1 then raise Not_found else r
  with :? System.ArgumentException -> invalid_arg "String.index_from" 
  
let rindex_from (s:string) (start:int) (c:char) =  
  try let r =  s.LastIndexOf(c,start) in if r = -1 then raise Not_found else r
  with :? System.ArgumentException -> invalid_arg "String.rindex_from" 
  

let index (s:string) (c:char) =  index_from s 0 c
let rindex (s:string) (c:char) =  rindex_from s (length s - 1) c

let contains_between (s:string) (start:int) (stop:int) (c:char) =  
  try s.IndexOf(c,start,(stop-start+1)) <> -1
  with :? System.ArgumentException -> invalid_arg "String.contains_between" 
    

let contains_from (s:string) (start:int) (c:char) =  
  let stop = length s - 1 in 
  try s.IndexOf(c,start,(stop-start+1)) <> -1
  with :? System.ArgumentException -> invalid_arg "String.contains_from" 

let rcontains_from (s:string) (stop:int) (c:char) =  
  let start = 0 in
  try s.IndexOf(c,start,(stop-start+1)) <> -1
  with :? System.ArgumentException -> invalid_arg "String.rcontains_from" 
  
let contains (s:string) (c:char) =  contains_from s 0 c

let uppercase (s:string) =  s.ToUpper()
let lowercase (s:string) =  s.ToLower()

let capitalize (s:string) =  
  if length s = 0 then invalid_arg "String.capitalize";
  concat "" [uppercase (sub s 0 1); sub s 1 (length s - 1)]

let uncapitalize (s:string) =  
  if length s = 0 then invalid_arg "String.uncapitalize";
  concat "" [lowercase (sub s 0 1); sub s 1 (length s - 1)]
