(*==========================================================================
 * Sys: Basic system operations (for ML compatibility)
 *
 * This module is only included to make it possible to cross-compile 
 * code with other ML compilers.  It may be deprecated and/or removed in 
 * a future release. You may wish to use .NET functions directly instead. 
 *
 * (c) Microsoft Corporation 2004.  All code and documentation in this file 
 * has been written 'from scratch'.  The interface to the module 
 * is similar to that found in versions of other ML implementations, 
 * but is not an exact match.  The type signatures in this interface
 * are an edited version of those generated automatically by running 
 * "bin\fsc.exe -i" on the implementation file.
 *=========================================================================*)

module Microsoft.FSharp.MLLib.Sys

(* --------------------------------------------------------------------
 * "argv" gives the command line arguments as returned by
 * System.Environment.GetCommandLineArgs().
 *
 * "file_exists s" returns true if a file currently exists,
 * using System.IO.File.Exists(s).
 *
 * "getenv s" gets the value of an environment variable, raising
 * "Not_found" if the variable is not defined.
 *
 * "remove s" deletes a file using System.IO.File.Delete(s).
 *
 * "rename s1 s2" renames a file on disk.
 * -------------------------------------------------------------------- *)
 
val argv: string array
val file_exists: string -> bool
val getenv: string -> string
val remove: string -> unit
val rename: string -> string -> unit

(* --------------------------------------------------------------------
 * "chdir p" sets the current working directory for the process.
 * "getcwd" returns the current working directory for the process.
 * -------------------------------------------------------------------- *)
 
val chdir: string -> unit
val getcwd: unit -> string

(* --------------------------------------------------------------------
 * Warning: 'command' currently attempts to execute the string using 
 * the 'cmd.exe' shell processor.  If it is not present on the system 
 * then the operation will fail.  Use System.Diagnostics.Process 
 * directly to run commands in a portable way, which involves specifying 
 * the program to run and the arguments independently.
 * -------------------------------------------------------------------- *)
 
val command: string -> int
 
(* --------------------------------------------------------------------
 * Misc.
 * -------------------------------------------------------------------- *)
 
val executable_name: string
val word_size: int

(* --------------------------------------------------------------------
 * [time] attempts to returns the processor time used by the main 
 * thread of the application.
 * -------------------------------------------------------------------- *)
 
val time: unit -> float
