(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.UInt32

let compare (x:uint32) y = Pervasives.compare x y

let zero = 0ul
let one = 1ul
let minus_one = (# "neg" 1ul : uint32)
let neg (x:uint32) =  (# "neg" x : uint32)
let add (x:uint32) (y:uint32) = (# "add" x y : uint32)
let sub (x:uint32) (y:uint32) = (# "sub" x y : uint32)
let mul (x:uint32) (y:uint32) = (# "mul" x y : uint32)
let div (x:uint32) (y:uint32) = (# "div.un" x y : uint32)
let rem (x:uint32) (y:uint32) = (# "rem.un" x y : uint32)
let succ (x:uint32) = add x one
let pred (x:uint32) = sub x one
let abs (x:uint32) = if x < zero then neg x else x
let max_int = 0xFFFFFFFFul
let min_int = 0ul 
let logand (x:uint32) (y:uint32) = (# "and" x y : uint32)
let logor (x:uint32) (y:uint32) = (# "or" x y : uint32)
let logxor (x:uint32) (y:uint32) = (# "xor" x y : uint32)
let lognot (x:uint32) = (# "not" x : uint32)
let shift_left (x:uint32) (n:int) =  (# "shl" x n : uint32)
let shift_right (x:uint32) (n:int) =  (# "shr.un" x n : uint32)
let of_int (n:int) =  (# "conv.u4" n : uint32)
let to_int (x:uint32) = (# "conv.i4" x : int)
let of_int32 (n:int32) =  (# "conv.u4" n : uint32)
let to_int32 (x:uint32) = (# "conv.i4" x : int32)

let of_float (f:float) =  (# "conv.u4" f : uint32)
let to_float (x:uint32) =  (# "conv.r8" x : float)

let of_string (s:string) = 
 try
  if (String.length s >= 3 & s.[0] = '-' & s.[1] = '0' & s.[2] = 'x') then 
    System.UInt32.Parse(String.sub s 3 (String.length s - 3), 
                        System.Globalization.NumberStyles.AllowHexSpecifier,
                        System.Globalization.CultureInfo.InvariantCulture)
  else if (String.length s >= 2 & s.[0] = '0' & s.[1] = 'x') then
    System.UInt32.Parse(String.sub s 2 (String.length s - 2), 
                        System.Globalization.NumberStyles.AllowHexSpecifier,
                        System.Globalization.CultureInfo.InvariantCulture)
  else 
    System.UInt32.Parse(s,System.Globalization.CultureInfo.InvariantCulture)
 with :? System.FormatException -> failwith "UInt32.of_string"

let to_string (x:uint32) = (box x).ToString()

(* Yes folks, the only verifiable way to get the bits from a float *)
(* is to blast it to a stream then read it back in... *)
let bits_of_float32 (x:float32) = 
   let ms = new System.IO.MemoryStream() in 
   let bw = new System.IO.BinaryWriter(ms) in 
   bw.Write(x); 
   ms.Flush();
   let br = new System.IO.BinaryReader(ms) in 
   ignore(ms.Seek(0L, System.IO.SeekOrigin.Begin)); 
   let res = br.ReadUInt32() in 
   br.Close(); 
   res

let float32_of_bits (x:uint32) = 
   let ms = new System.IO.MemoryStream() in 
   let bw = new System.IO.BinaryWriter(ms) in 
   bw.Write(x); 
   ms.Flush();
   let br = new System.IO.BinaryReader(ms) in 
   ignore(ms.Seek(0L, System.IO.SeekOrigin.Begin)); 
   let res = br.ReadSingle() in 
   br.Close(); 
   res


let float32_to_float (x:float32) = (# "conv.r8" x : float)
let float_to_float32 (x:float) = (# "conv.r4" x : float32)

let float_of_bits x = float32_to_float (float32_of_bits x)
let bits_of_float x = bits_of_float32 (float_to_float32 x)

let of_unativeint (f:unativeint) =  (# "conv.u4" f : uint32)
let to_unativeint (x:uint32) =  (# "conv.u" x : unativeint)

let of_uint64 (f:uint64) =  (# "conv.u4" f : uint32)
let to_uint64 (x:uint32) =  (# "conv.u8" x : uint64)



let of_float32 (f:float32) =  (# "conv.u4" f : uint32)
let to_float32 (x:uint32) =  (# "conv.r4" x : float32)
