(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.UInt64

let compare (x:uint64) y = Pervasives.compare x y

let zero = (# "ldc.i8 0" : uint64) 
let one = (# "ldc.i8 1" : uint64) 
let minus_one = (# "ldc.i8 -1" : uint64) 
let neg (x:uint64) =  (# "neg" x : uint64) 
let add (x:uint64) (y:uint64) = (# "add" x y : uint64) 
let sub (x:uint64) (y:uint64) = (# "sub" x y : uint64) 
let mul (x:uint64) (y:uint64) = (# "mul" x y : uint64)
let div (x:uint64) (y:uint64) = (# "div.un" x y : uint64)
let rem (x:uint64) (y:uint64) = (# "rem.un" x y : uint64)
let succ (x:uint64) = (# "ldc.i8 1 add" x : uint64)
let pred (x:uint64) = (# "ldc.i8 1 sub" x : uint64)
let abs (x:uint64) = if x < zero then neg x else x
let max_int = 0xFFFFFFFFFFFFFFFFUL
let min_int = 0x0UL
let logand (x:uint64) (y:uint64) = (# "and" x y : uint64)
let logor (x:uint64) (y:uint64) = (# "or" x y : uint64)
let logxor (x:uint64) (y:uint64) = (# "xor" x y : uint64)
let lognot (x:uint64) = (# "not" x : uint64)
let shift_left (x:uint64) (n:int) =  (# "shl" x n : uint64)
let shift_right (x:uint64) (n:int) =  (# "shr.un" x n : uint64)
let of_int (n:int) =  (# "conv.u8" n : uint64)
let to_int (x:uint64) = (# "conv.i4" x : int)
let of_uint32 (n:uint32) =  (# "conv.u8" n : uint64)
let to_uint32 (x:uint64) = (# "conv.u4" x : uint32)
let of_int64 (n:int64) =  (# "conv.u8" n : uint64)
let to_int64 (x:uint64) = (# "conv.i8" x : int64)
let of_unativeint (n:unativeint) =  (# "conv.u8" n : uint64)
let to_unativeint (x:uint64) = (# "conv.u" x : unativeint)
let of_float (f:float) =  (# "conv.u8" f : uint64)
let to_float (x:uint64) =  (# "conv.r8" x : float)
let of_string (s:string) = 
 try
  if (String.length s >= 3 & s.[0] = '-' & s.[1] = '0' & s.[2] = 'x') then 
    System.UInt64.Parse(String.sub s 3 (String.length s - 3), 
                        System.Globalization.NumberStyles.AllowHexSpecifier,
                        System.Globalization.CultureInfo.InvariantCulture)
  else if (String.length s >= 2 & s.[0] = '0' & s.[1] = 'x') then
    System.UInt64.Parse(String.sub s 2 (String.length s - 2), 
                        System.Globalization.NumberStyles.AllowHexSpecifier,
                        System.Globalization.CultureInfo.InvariantCulture)
  else 
    System.UInt64.Parse(s,System.Globalization.CultureInfo.InvariantCulture)
 with :? System.FormatException -> failwith "Int32.of_string"
let to_string (x:uint64) = (box x).ToString()

(* Yes folks, the only verifiable way to get the bits from a float *)
(* is to blast it to a stream then read it back in... *)
let bits_of_float (x:float) = 
   let ms = new System.IO.MemoryStream() in 
   let bw = new System.IO.BinaryWriter(ms) in 
   bw.Write(x); 
   ms.Flush();
   let br = new System.IO.BinaryReader(ms) in 
   ignore(ms.Seek(Int64.of_int 0, System.IO.SeekOrigin.Begin)); 
   let res = br.ReadUInt64() in 
   br.Close(); 
   res

let float_of_bits (x:uint64) = 
   let ms = new System.IO.MemoryStream() in 
   let bw = new System.IO.BinaryWriter(ms) in 
   bw.Write(x); 
   ms.Flush();
   let br = new System.IO.BinaryReader(ms) in 
   ignore(ms.Seek(Int64.of_int 0, System.IO.SeekOrigin.Begin)); 
   let res = br.ReadDouble() in 
   br.Close(); 
   res


let of_float32 (f:float32) =  (# "conv.u8" f : uint64)
let to_float32 (x:uint64) =  (# "conv.r4" x : float32)
