(*==========================================================================
 * UInt64: ML-like operations on 64-bit System.UInt64 floating point numbers.
 *
 * (c) Microsoft Corporation 2004.  All code and documentation in this file 
 * has been written 'from scratch'.  The interface to the module 
 * is similar to that found in versions of other ML implementations, 
 * but is not an exact match.  The type signatures in this interface
 * are an edited version of those generated automatically by running 
 * "bin\fsc.exe -i" on the implementation file.
 *=========================================================================*)

module Microsoft.FSharp.MLLib.UInt64

val zero: uint64
val one: uint64
val pred: uint64 -> uint64
val succ: uint64 -> uint64
val max_int: uint64
val min_int: uint64
val minus_one: uint64

val abs: uint64 -> uint64
val add: uint64 -> uint64 -> uint64
val div: uint64 -> uint64 -> uint64
val mul: uint64 -> uint64 -> uint64
val neg: uint64 -> uint64
val rem: uint64 -> uint64 -> uint64
val sub: uint64 -> uint64 -> uint64

val compare: uint64 -> uint64 -> int

val logand: uint64 -> uint64 -> uint64
val lognot: uint64 -> uint64
val logor: uint64 -> uint64 -> uint64
val logxor: uint64 -> uint64 -> uint64
val shift_left: uint64 -> int -> uint64
val shift_right: uint64 -> int -> uint64

val of_float: float -> uint64
val to_float: uint64 -> float

val of_float32: float32 -> uint64
val to_float32: uint64 -> float32

(* Conversions to int are included because int is the most convenient *)
(* integer type to use from F#.  Otherwise conversions are either to *)
(* integers of the same size or same sign *)
val of_int: int -> uint64
val to_int: uint64 -> int

val of_uint32: uint32 -> uint64
val to_uint32: uint64 -> uint32

val of_int64: int64 -> uint64
val to_int64: uint64 -> int64

val of_unativeint: unativeint -> uint64
val to_unativeint: uint64 -> unativeint

val of_string: string -> uint64
val to_string: uint64 -> string

val float_of_bits: uint64 -> float
val bits_of_float: float -> uint64

