(* (c) Microsoft Corporation. All rights reserved *)

module Microsoft.FSharp.MLLib.Vector

type 'a Vector = Vector of 'a array
type 'a vector = 'a Vector
type 'a t = 'a Vector // For use when not opening module, e.g. Vector.t

let length (Vector x) = Array.length x
let get (Vector x) i = Array.get x i
let init (n:int) (f: int -> 'a) = Vector(Array.init n f)
let concat vs = Vector(Array.concat (List.map (fun (Vector x) -> x) vs))
let append (Vector arr1) (Vector arr2) = Vector(Array.append arr1 arr2)
let sub (Vector a) s l = Vector(Array.sub a s l)
let to_list (Vector a) = Array.to_list a  
let of_list x = Vector(Array.of_list x)
let iter f (Vector arr) = Array.iter f arr
let map f (Vector arr) = Vector(Array.map f arr)
let iteri f (Vector arr) = Array.iteri f arr
let mapi f (Vector arr) = Vector(Array.mapi f arr)
let fold_left f acc (Vector arr) = Array.fold_left f acc arr
let fold_right f (Vector arr) acc = Array.fold_right f arr acc
let of_stream s = Vector(Stream.to_array s)
let to_stream (Vector s) = Stream.of_array s

#if CLI_FRAMEWORK_POST_WHIDBEY_BETA1
open System.Collections.Generic

let get_IEnumerator (Vector s) = Array.get_IEnumerator(s)
        
// Create a new object to ensure underlying array may not be mutated by a backdoor cast 
let to_IEnumerable s = 
  { new IEnumerable<_> with 
      GetEnumerator() = get_IEnumerator s
    interface System.Collections.IEnumerable
      with GetEnumerator() = (get_IEnumerator s :> System.Collections.IEnumerator) }

// Create a new object to ensure underlying array may not be mutated by a backdoor cast 
let to_ICollection (Vector s) = 
  let c : ICollection<'a> = Array.to_ICollection(s) in
  { new ICollection<'a> 
      with Add(x) = raise (new System.NotSupportedException("ReadOnlyCollection"));
      and Clear() = raise (new System.NotSupportedException("ReadOnlyCollection"));
      and Remove(x) = raise (new System.NotSupportedException("ReadOnlyCollection"));
      and Contains(x) = c.Contains(x)
      and CopyTo(arr,i) = c.CopyTo(arr,i)
      and get_IsReadOnly() = true
      and get_Count() = c.Count
    interface IEnumerable<'a>
      with GetEnumerator() = (c :> IEnumerable<'a>).GetEnumerator() 
    interface System.Collections.IEnumerable
      with GetEnumerator() = (c :> System.Collections.IEnumerable).GetEnumerator() }


#endif


#if CLI_FRAMEWORK_EXACTLY_WHIDBEY_BETA1

open System.Collections.Generic

let get_IEnumerator (Vector s) = Array.get_IEnumerator(s)
        
// Create a new object to ensure underlying array may not be mutated by a backdoor cast 
let to_IEnumerable s = 
  { new IEnumerable<_> with 
      GetEnumerator() = get_IEnumerator s }

// Create a new object to ensure underlying array may not be mutated by a backdoor cast 
let to_ICollection (Vector s) = 
  let c = Array.to_ICollection(s) in
  { new ICollection<_> 
      with Add(x) = raise (new System.NotSupportedException("ReadOnlyCollection"));
      and Clear() = raise (new System.NotSupportedException("ReadOnlyCollection"));
      and Remove(x) = raise (new System.NotSupportedException("ReadOnlyCollection"));
      and Contains(x) = c.Contains(x)
      and CopyTo(arr,i) = c.CopyTo(arr,i)
      and get_IsReadOnly() = true
      and get_Count() = c.Count
      and GetEnumerator() = c.GetEnumerator() }

#endif

#if GENERICS
let of_IEnumerable c = Vector(Array.of_IEnumerable c)

let of_ICollection c = Vector(Array.of_ICollection c)
  
let to_List (Vector l) = Array.to_List l
let of_List (l : List<'a>) = Vector(Array.of_List l)
  
#endif

let foreach x f  = iter f x
let transform x f  = map f x
let foreachi x f  = iteri f x
let transformi x f  = mapi f x
