// Copyright (c) Microsoft Corporation. All rights reserved.
// This sample code is provided "as is" without warranty of any kind. 
// We disclaim all warranties, either express or implied, including the 
// warranties of merchantability and fitness for a particular purpose. 

open System.IO
open Ast
open List
open Printf

open Lexing

let main() = 
  if Array.length Sys.argv <> 2 then begin 
    printf "usage: interp.exe <file>\n";
    exit 1;
  end;
  
  let stream = new StreamReader(Sys.argv.(1)) in 
  let myProg = 
    (* Create the lexer *)
    let lexbuf = Lexing.from_stream_reader stream in 
    (* Call the parser *)
    try Pars.start Lex.token lexbuf
    with e -> 
      let pos = lexbuf_curr_p lexbuf in 
      printf "error near line %d, character %d\n%s\n" pos.pos_cnum (pos.pos_cnum - pos.pos_bol) (e.ToString()); 
      exit 1 in
    
(* Now look at the resulting AST, e.g. count the number of top-level *)
(* statements, and then the overall number of nodes. *)
  printf "#stmts = %d\n" (List.length (match myProg with Prog l -> l));
  printf "running program...\n";
  Interp.prog myProg

do main()




