// Copyright (c) Microsoft Corporation. All rights reserved.
// This sample code is provided "as is" without warranty of any kind. 
// We disclaim all warranties, either express or implied, including the 
// warranties of merchantability and fitness for a particular purpose. 

open System
open System.IO
open System.Windows.Forms
open Printf

// Create a form and set some properties

let form = new Form()
do form.Text <- "My First F# Form"

let menu = form.Menu <- new MainMenu()
let mnuFile = form.Menu.MenuItems.Add("&File")

let mnuiOpen = 
  new MenuItem("&Open...", 
               new EventHandler(fun _ _ -> 
                 let d = new OpenFileDialog() in 
                 d.InitialDirectory <- "c:\\";
                 d.Filter <- "txt files (*.txt)|*.txt|All files (*.*)|*.*";
                 d.FilterIndex <- 2;
                 d.RestoreDirectory <- true;
                 if d.ShowDialog() = DialogResult.OK then 
                   begin match d.OpenFile() with 
                   | null -> printf "Ooops... Could not read the file...\n"
                   | s -> 
                       let r = new StreamReader(s) in 
                       printf "The first line of the file is: %s!\n" (r.ReadLine());
                       s.Close();
                   end;), 
               Shortcut.CtrlO)
do ignore(mnuFile.MenuItems.Add(mnuiOpen))

// Run the main code. The attribute marks the startup application thread as "Single 
// Thread Appartment" mode, which is necessary for GUI applications. 
[<STAThread>]    
do Application.Run(form)
