// Copyright (c) Microsoft Corporation. All rights reserved.
// This sample code is provided "as is" without warranty of any kind. 
// We disclaim all warranties, either express or implied, including the 
// warranties of merchantability and fitness for a particular purpose. 

(* Echo Client: connects to a server (presumed to be on the local host) and *)
(* sends it data, presuming it to be an echo server which bounces data back.  *)

open System.Net
open System.Net.Sockets

let _ = 
  try 
    Printf.printf "client created...\n";
    if Array.length Sys.argv < 2 then (Printf.eprintf "usage: client <port>"; exit 1); 
    let csock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP) in 
    let port = int_of_string (Sys.argv.(1)) in 
    Printf.printf "client connecting on port %d...\n" port;
    csock.Connect(new System.Net.IPEndPoint(System.Net.IPAddress.Parse "127.0.0.1", port));
    let buf = Bytearray.create 64 in
    while (true) do 
      let data = System.Console.In.ReadLine() in 
      if data.Length > 0 then 
        let dataToSend = System.Text.Encoding.ASCII.GetBytes(data) in 
        let n = csock.Send(dataToSend) in 
        Printf.printf "client wrote %d bytes\n" n;
        let n = csock.Receive(buf) in 
        Printf.printf "client read: %s\n" (System.Text.Encoding.ASCII.GetString(buf,0,n));
    done;
    Printf.printf "client closing...\n";
    csock.Close()
  with e when (e :? Sockets.SocketException) or
              (e :? System.IO.IOException) -> 
      Printf.printf "server gone!\n";
      Printf.printf "Error was: %s\n" (e.ToString())


