// Copyright (c) Microsoft Corporation. All rights reserved.
// This sample code is provided "as is" without warranty of any kind. 
// We disclaim all warranties, either express or implied, including the 
// warranties of merchantability and fitness for a particular purpose. 

(* Echo Server: opens a service on the localhost which bounces data straight back  *)

open System.Net
open System.Net.Sockets

let thread_start f = 
  (new System.Threading.Thread(new System.Threading.ThreadStart(f))).Start()

let _  =
  Printf.printf "server starting up...\n";
  let sock = 
    new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP) in 
   
  let addr = IPAddress.Parse("127.0.0.1") in
  let port = 0 in 
  sock.Bind(new IPEndPoint(addr, port));
  let backlog = 2 in (* number of incoming connections that can be queued for acceptance*)
  sock.Listen(backlog);
  let port = (sock.LocalEndPoint :?> IPEndPoint).Port in 
  Printf.printf "server accepting on port %d...\n" port;
  while (true) do
    let csock = sock.Accept() in 
    Printf.printf "the server has accepted a client on port %d...\n" port;
    thread_start(fun () -> 
      let total = ref 0 in 
      try 
        let buf = Bytearray.create 64 in
        let stream = new NetworkStream(csock) in 
        while true do
          Printf.printf "server reading...\n";
          let nread = stream.Read(buf,0,64) in 
          Printf.printf "server read %d...\n" nread;
          stream.Write(buf,0,nread);
          total := !total + nread;
        done;
      with e when (e :? Sockets.SocketException) or
                  (e :? System.IO.IOException) -> 
        Printf.printf "client gone: server read %d bytes from that client\n" !total;
        Printf.printf "Error was: %s\n" (e.ToString()); 
        flush stdout)
  done


