// Copyright (c) Microsoft Corporation. All rights reserved.
// This sample code is provided "as is" without warranty of any kind. 
// We disclaim all warranties, either express or implied, including the 
// warranties of merchantability and fitness for a particular purpose. 

(* This program demonstrates using a classic COM object from F#. *)

open System
open System.Runtime.InteropServices

(* Collects the filename of an AVI to show then create an  *)
(* instance of the Quartz classic COM object.  To show the AVI, the program calls *)
(* RenderFile and Run on IMediaControl.  Quartz uses its own thread and window *)
(* to display the AVI.  The main thread blocks on a ReadLine until the user hits *)
(* enter.  *)

let processOneFile file =
  let mc = new QuartzTypeLib.FilgraphManagerClass() in  (* QuartzTypeLib.IMediaControl *)
  begin
    try
      mc.RenderFile(file);
      mc.Run();
    with  e -> prerr_endline (Printexc.to_string e)
  end;
  Console.WriteLine("Hit Enter to continue/exit.");
  let _ = Console.ReadLine() in 
  ()

let main() = 
   if Sys.argv.Length = 1 then Console.WriteLine ("play [file1.avi [file2.avi ...] ]");
   Arg.parse [] processOneFile "play [file1.avi [file2.avi ...] ]"
   
// Run the main code. The attribute marks the startup application thread as "Single 
// Thread Appartment" mode, which is necessary for GUI applications. 
[<STAThread>]    
do main()


